# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.



from qt import *
from dprint import *

class Menu:
    """This is a class used by PlotRepository to build the Plot menu
    after the plots have been scanned and identified by the Plot Repository."""

    def __init__(self, menu = None, name = ""):
        """Constructor
        Params:
               - menu : QPopupMenu - The menu to be inserted at the top of this menu
               - name : String - The name that this menu should have when inserting
               it into another menu or a toolbar"""
        
        self.menu = menu
        self.items = []
        self.subMenus = {}
        self.name = name

    def printNice(self, tabs = 0):
        """
        Params:
               - tabs : int - The number of tabs that should be inserted before
               printing anythin. Just useful when called recurively (default = 0)

        Prints a nice representation of the menu structure to stdout"""
        
        print "\t"*tabs + self.name + ":"

        for item in self.items:
            print "\t"*(tabs+1) + item

        for menu in self.subMenus:
            self.subMenus[menu].printNice(tabs+1)

    def addSubMenu(self, menu, menuText):
        """
        Params:
               - menu : QPopupMenu - The menu that should be inserted into this menu
               - menuText : String - The name of the menu

        Inserts a submenu into this menu"""
        
        self.subMenus[menuText] = Menu(menu, menuText)
        self.menu.insertItem(menuText, menu)
        
        dprint(flags.GUI.PLOTS.BUILD, "Adding submenu " + menuText + " to " + self.name)
        
    def addItem(self, text, index, slot):
        """
        Params:
               - text : String - The text of the item to be added
               - index : int - a number given to the item when added
               to the menu. This is just a tracking/id number.
               - slot : Function - The slot to be fired when the item is finally
               selcted in a menu
        Adds a menu item to this menu. Use addSubMenu if you want to add another
        menu into this one"""
        
        self.items.append(text)
        id = self.menu.insertItem(text, slot)
        self.menu.setItemParameter(id, index)
        
        dprint(flags.GUI.PLOTS.BUILD,  "Adding item " + text +
               " with id: " + str(index) + " to menu...")
        

    def getSubMenu(self, menuText):
        """
        Params:
               - menuText : String - The text of the menu that we want.

        Return:
               - Menu - If a menu with the right text exists, otherwise None"""
        
        for menu in self.subMenus:

            if menu == menuText:
                return self.subMenus[menuText]

        return None

    def menuExists(self, menuText):
        """
        Params:
               - menuText : String - the text of the menu that should be checked

        Checks whether this menu has a submenu with text menuText"""
        
        for sub in self.subMenus:

            if sub == menuText:

                return 1

        return 0

    def itemExists(self, itemText):
        """
        Params:
               - itemText : String - The text of the item we want to find

        Checks wether this menu has an item with the text itemText"""
        
        for item in self.items:

            if item == itemText:

                return 1

        return 0

    
        
        
