# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.



import glob
import re
import sys
import os
import string
from Library.Util.Config import *
from Library.Util.unrepr import unrepr
from qt import QPoint

class NodeConfig:
    """Class used to take care of node confuration"""


    def __init__(self):
        self.type = ""
        self.image = ""
        self.points = []
        self._inpoints = []
        self._outpoints = []

    def get_outpoints(self):
        """Get list of exit points on this node.

        Each point is of format [start, end], where start and end are
        of type QPoint.
        """
        return self._outpoints

    def get_inpoints(self):
        """Get list of entry points on this node.

        Each point is of format [start, end], where start and end are
        of type QPoint.
        """
        return self._inpoints

    def readConfig(self, node):      
        self.type = ""
        self.image = ""
        self.points = []
        self.nodeStyle = Config.nodeStyle


        try:
            first, second = tuple(node.split("."))
            bla, node = tuple(second.split(":"))
        except:
            pass
            
        # <--- added test to check wether file exists or not. used silly default value if not    
        cfg = None

        path = os.path.join(Config.nodeconfdir, str(node) + ".txt")
        if not os.path.exists(path):
            node = "Unknown"

        cfg = open(os.path.join(Config.nodeconfdir, str(node) + ".txt"))

        inpoints = {}
        outpoints = {}

        # remove whitespace
        lines = map(string.strip, cfg.readlines())
        # filter out empty lines and comments
        lines = filter(lambda l : l and not l.startswith("#"), lines)

        for line in lines:
            item, value = map(string.strip, line.split("="))

            if item.startswith("inpoint"):
                points = value.split(":")
                start, end = map(unrepr, value.split(":"))
                index = int(item[len("inpoint"):])
                inpoints[index] = (QPoint(*start), QPoint(*end))

            elif item.startswith("outpoint"):
                points = value.split(":")
                start, end = map(unrepr, value.split(":"))
                index = int(item[len("outpoint"):])
                outpoints[index] = (QPoint(*start), QPoint(*end))

            elif item.startswith("point"):
                # value should look something like '[0, 1] : [2, 3]'
                value = re.sub("\s", "", value) #remove whitepsace
                start, end = tuple(value.split(":")) # split the two points
                start = re.sub("\[", "", re.sub("\]", "", start)) 
                end = re.sub("\[", "", re.sub("\]", "", end))
                x1, y1 = tuple(start.split(",")) 
                x2, y2 = tuple(end.split(","))
                self.points.append([QPoint(eval(x1), eval(y1)), QPoint(eval(x2), eval(y2))])
                
            elif item == "image":
                # need to test if the image is there or not
                if not os.path.exists(os.path.join(Config.pixmapdir, str(value) + ".png")):
                    print value, "does not exist in", Config.pixmapdir, "using Unknown.png instead"
                    value = "Unknown"
                if self.nodeStyle:
                    name, prefix = value.split(".")
                    name  = name + "-" + str(self.nodeStyle)
                    value = name + "." + prefix
                    if not os.path.exists(os.path.join(Config.pixmapdir,  str(value) + ".png")):
                        print value, "does not exist in", Config.pixmapdir, "using Unknown.png instead"
                        value = "Unknown"

                value = os.path.join(Config.pixmapdir, str(value))
                setattr(self, item, value)
                
            elif value == "None":
                value = None
                setattr(self, item, value)
                
            else:
                setattr(self, item, value)

        inpoint_keys = inpoints.keys()
        inpoint_keys.sort()
        for key in inpoint_keys:
            self._inpoints.append(inpoints[key])

        outpoint_keys = outpoints.keys()
        outpoint_keys.sort()
        for key in outpoint_keys:
            self._outpoints.append(outpoints[key])

        # points count as both input and output
        for point in self.points:
            self._inpoints.append(point)
            self._outpoints.append(point)

        return 1
    
    def getImage(self):
        return self.image
        
    def getPoints(self):
        return self.points
