# -*- coding: utf-8 -*-
# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.

from GUI.guinode import GUINode
from qtui import QWidgetFactory
from qt import *
import os
from Library.Util import lists
from Library.Util.Config import Config
from GUI.NodeConfig import NodeConfig

class InputGUINode(GUINode):
    def __init__(self, parent, node):
        self._node = node
        self._parent = parent
        GUINode.__init__(self, parent, node)
        self.changeIcon()
        # Created later to defer resizing application before fully created
        self._input_node_dialog = None

    def _show_input_node(self):
        if not self._input_node_dialog:
            self._input_node_dialog = InputNodeDialog(self._node, self, self._parent)
        self._input_node_dialog.show()


    def nodeDialog(self):
        self._show_input_node()
        
     
    def changeIcon(self):
        nodename = self._node.getFileReaderClassName()
        
        nodeConfig = NodeConfig()
        nodeConfig.readConfig(nodename)
        imageURL = (nodeConfig.getImage()) + ".png"
        
        if imageURL.find("Unknown") > 0:
            nodeConfig.readConfig("InputNode")
            imageURL = (nodeConfig.getImage()) + ".png"
            
        self.getCanvasNode().setImage(imageURL)


class InputNodeDialog:
    """This window will contain the inputnode.
    """


    def __init__(self, node, gui_node, parent):
        factory = QWidgetFactory()
        self._widget = factory.create(os.path.join(Config.uidir, "multipleinputnodewidget.ui"), None, parent, None)
        self._gui_node = gui_node
        self._node = node
        self._parent = parent 
        self._ready_input_node_gui()
        self._sep = None
        self._sep_gui = None

    def show(self):
        self._possible_ports = []
        self._chosen_ports = []
        self.set_saved_values()
        
        self._widget.show()
                

    def _ready_input_node_gui(self):
        """Readies the inputnode dialog with functionality that cannot be
        added in the ui-file
        """

        dialog = self._widget

        #Getting childs
        self._line_edit_filename = dialog.child("line_edit_filename")
        self._push_button_browse = dialog.child("push_button_browse")
        self._list_view_possible = dialog.child("list_view_output_ports_possible")
        self._list_view_chosen = dialog.child("list_view_output_ports_chosen")
        self._push_button_add = dialog.child("push_button_output_port_add")
        self._push_button_remove = dialog.child("push_button_output_port_remove")
        self._push_button_ok = dialog.child("push_button_ok")
        self._push_button_cancel = dialog.child("push_button_cancel")
        self._combo_box_filetype = dialog.child("combo_box_filetype")
        self._text_edit_file_description = dialog.child("text_edit_filetype_description")
        self._sep_button= dialog.child("sep_button")
        self._sep_button.hide()
        dialog.connect(self._sep_button, SIGNAL("clicked()"), self._sep_button_clicked)
        #Connecting signals on a widget to slots
        dialog.connect(self._push_button_browse, SIGNAL("clicked()"), self._push_button_browse_clicked)
        dialog.connect(self._push_button_add, SIGNAL("clicked()"), self._push_button_inputport_add_clicked)
        dialog.connect(self._push_button_remove, SIGNAL("clicked()"), self._push_button_inputport_remove_clicked)
        dialog.connect(self._push_button_ok, SIGNAL("clicked()"), self._push_button_ok_clicked)
        dialog.connect(self._push_button_cancel, SIGNAL("clicked()"), self._push_button_cancel_clicked)
        dialog.connect(self._line_edit_filename, SIGNAL("textChanged(const QString &)"),\
            self._filename_changed)
        dialog.connect(self._combo_box_filetype, SIGNAL("activated(const QString &)"),\
            self._filetype_changed)
        
        #Some static preferences
        self._push_button_add.setPixmap(QPixmap(os.path.join(Config.iconsdir, "arrowForwardPic.xpm")))
        self._push_button_remove.setPixmap(QPixmap(os.path.join(Config.iconsdir, "arrowBackPic.xpm")))
        self._list_view_possible.setSortColumn(-1)
        self._list_view_possible.setColumnAlignment(0, Qt.AlignCenter)
        self._list_view_possible.setColumnAlignment(1, Qt.AlignCenter)
        self._list_view_chosen.setSortColumn(-1)
        self._list_view_chosen.setColumnAlignment(0, Qt.AlignCenter)
        self._list_view_chosen.setColumnAlignment(1, Qt.AlignCenter)
        
        #Insert list of file types
        self._fileTypeList = self._node.getFileTypeList()
        self._combo_box_filetype.insertStrList(self._fileTypeList)
        
    
    def _push_button_browse_clicked(self):
        file_filters = self._node.getFilters()
        filename = QFileDialog.getOpenFileName("", file_filters, self._widget,\
            "open_file_dialog", "Choose a file to read...")
        if filename:
            self._line_edit_filename.setText(str(filename))
            
            
    def _filename_changed(self, filename): # New
        filename = str(filename)

        # Get current filetype
        index = self._combo_box_filetype.currentItem()

        if self._node.file_exists(filename):
            new_filetype = self._node.get_compatible_filetype(filename)
        
            # Keep the old filetype if the new one isn't recognized
            if new_filetype != self._fileTypeList[0]: # "Unknown"
                index = self._fileTypeList.index(new_filetype)

        if index < 0:
            index = 0
        self._combo_box_filetype.setCurrentItem(index)
        self._filetype_changed(self._fileTypeList[index])


    def _filetype_changed(self, filetype): # New
        self.clear_list_views()
        filename = str(self._line_edit_filename.text())
        filetype = str(filetype)
        filetype_desc = self._node.get_filetype_description(filetype)
        self._text_edit_file_description.setText(filetype_desc)

        if self._node.file_exists(filename):
            possible_ports = self._node.get_possible_output_ports(filename, filetype)
            chosen_ports = self._node.get_chosen_output_ports(filename, filetype)
            default_ports = self._node.get_default_output_ports(filetype)
            if not possible_ports and not default_ports and not chosen_ports:
                QMessageBox.warning(self._widget, "Warning", "No variables found.", "&Ok","", "", 0, 1)
            
            self.display_ports(possible_ports, chosen_ports, default_ports)
        if self._combo_box_filetype.currentText() == "Textfile":
            self._sep_button.show()
        else:
            self._sep_button.hide()

    def _sep_button_clicked(self):
        if not self._sep:
            self._sep = self._node.textfile_separator()
        self._sep_gui = SeparatorGUI(self._sep, self._parent)

    def _insert_port_in_list_view(self, list_view, port_name, port_info):
        if (list_view.childCount()):
            QListViewItem(list_view, list_view.lastItem(), port_name, port_info)
        else:
            QListViewItem(list_view, port_name, port_info)
        
                        
    def _move_items(self, from_list, to_list): # New
        iterator = QListViewItemIterator(from_list, QListViewItemIterator.Selected)
        
        item_list = []
        while iterator.current():
            item = iterator.current()
            item_list.append(item)
            name = item.text(0)
            info = item.text(1)
            iterator += 1

        for item in item_list:
            from_list.takeItem(item)
            self._insert_port_in_list_view(to_list, item.text(0), item.text(1))

        to_list.clearSelection()
        
        
    def _push_button_inputport_add_clicked(self):
           self._move_items(self._list_view_possible, self._list_view_chosen) 
        
    def _push_button_inputport_remove_clicked(self):
        self._move_items(self._list_view_chosen, self._list_view_possible) 
        
        
    def _push_button_ok_clicked(self):
        
        filename = str(self._line_edit_filename.text())
        filetype = str(self._combo_box_filetype.currentText())
        if filename and not self._node.file_exists(filename):
            QMessageBox.warning(self._widget, "File not found", "The file:\n" + filename + "\nwas not found.", "&Ok","", "", 0, 1)

        self._node.set_filename(filename)
        self._node.set_filetype(filetype)
        
        iterator = QListViewItemIterator(self._list_view_chosen, QListViewItemIterator.Visible)
        currently_chosen = []

        while iterator.current():
            item = iterator.current()
            port = (str(item.text(0)), str(item.text(1))) # name, info
            currently_chosen.append(port)
            if port not in self._chosen_ports:
                # we can add a node several times without problems, this
                # should just update the description
                self._node.add_output_port(port[0], "", port[1], "")
             
            iterator += 1

        first = lambda seq : seq[0]

        chosen_port_names = map(first, currently_chosen)
        existing_port_names = map(first, self._chosen_ports)

        difference = lists.filter_list(existing_port_names, chosen_port_names)
        
        for port in difference:
            if port:
                self._node.delete_port(port) # name

        self._gui_node.changeIcon()
        self.close()

        
    def _push_button_cancel_clicked(self):
        self.close()
        
        
    def close(self):
        self._widget.close()

    
    def clear_list_views(self):
        self._list_view_possible.clear()
        self._list_view_chosen.clear()


    def display_ports(self, possible_ports, chosen_ports, default_ports): # New
        self.clear_list_views()
        
        for port in possible_ports:
            name, info = port
            self._insert_port_in_list_view(self._list_view_possible, name, info)

        if default_ports:
            for port in default_ports:
                name, info = port
                self._insert_port_in_list_view(self._list_view_chosen, name, info)
        elif chosen_ports:
            for port in chosen_ports:
                name, info = port
                self._insert_port_in_list_view(self._list_view_chosen, name, info)
        

    def set_saved_values(self): # New
        """This method restores previous saved values."""

        filename = self._node.get_current_filename()
        filetype = self._node.get_current_filetype()
        description = self._node.get_filetype_description(filetype)
        self._possible_ports = self._node.get_possible_output_ports(filename, filetype)
        default_ports = self._node.get_default_output_ports(filetype)
        self._chosen_ports = self._node.get_current_output_ports()

        # checks that the chosen ports returned truly exists in the file
        all_variables = self._node.get_all_variable_names_in_file(filename, filetype)
        new_chosen_ports = []
        if self._chosen_ports:
            for port in self._chosen_ports:
                portname, portdim = port
                if portname in all_variables:
                    new_chosen_ports.append((portname, portdim))
                else:
                    self._node.delete_port(portname)

        self._chosen_ports = new_chosen_ports
        
        if self._chosen_ports:
            for port in default_ports:
                if port not in self._chosen_ports:
                    self._possible_ports.append(port)
            default_ports = []
        
        self._line_edit_filename.setText(filename)
        filetype_index = self._fileTypeList.index(filetype)
        self._combo_box_filetype.setCurrentItem(filetype_index)
        self._text_edit_file_description.setText(description)

        self.display_ports(self._possible_ports, self._chosen_ports, default_ports)
        self._gui_node.changeIcon()
        if self._combo_box_filetype.currentText() == "Textfile":
            self._sep_button.show()
        else:
            self._sep_button.hide()
       
class SeparatorGUI:

    def __init__(self, separator, parent):
        
        self._sep = separator
        factory = QWidgetFactory()
        self._gui = factory.create(os.path.join(Config.uidir, "sep.ui"), None, parent, None)     
        gui = self._gui
        
        self._r0 = gui.child('R0')
        self._r1 = gui.child('R1')
        self._r2 = gui.child('R2')
        self._r3 = gui.child('R3')
        self._r4 = gui.child('R4')
        self._r5 = gui.child('R5')
        self._r6 = gui.child('R6')
        self._r7 = gui.child('R7')
        self.row = [self._r0, self._r1, self._r2, self._r3, self._r4, self._r5, self._r6, self._r7]
        self._c0 = gui.child('C0')
        self._c1 = gui.child('C1')
        self._c2 = gui.child('C2')
        self._c3 = gui.child('C3')
        self._c4 = gui.child('C4')
        self._c5 = gui.child('C5')
        self._c6 = gui.child('C6')
        self._c7 = gui.child('C7')
        self.col = [self._c0, self._c1, self._c2, self._c3, self._c4, self._c5, self._c6, self._c7]
        self._ok = gui.child('ok')
        self._cancel = gui.child('cancel')

        gui.connect(self._r0, SIGNAL('textChanged(const QString &)'), self._r0_text)
        gui.connect(self._r1, SIGNAL('toggled(bool)'), self._r1_toggle)
        gui.connect(self._r2, SIGNAL('toggled(bool)'), self._r2_toggle)
        gui.connect(self._r3, SIGNAL('toggled(bool)'), self._r3_toggle)
        gui.connect(self._r4, SIGNAL('toggled(bool)'), self._r4_toggle)
        gui.connect(self._r5, SIGNAL('toggled(bool)'), self._r5_toggle)
        gui.connect(self._r6, SIGNAL('toggled(bool)'), self._r6_toggle)
        gui.connect(self._r7, SIGNAL('toggled(bool)'), self._r7_toggle)
       
       
        gui.connect(self._c0, SIGNAL('textChanged(const QString &)'), self._c0_text)
        gui.connect(self._c1, SIGNAL('toggled(bool)'), self._c1_toggle)
        gui.connect(self._c2, SIGNAL('toggled(bool)'), self._c2_toggle)
        gui.connect(self._c3, SIGNAL('toggled(bool)'), self._c3_toggle)
        gui.connect(self._c4, SIGNAL('toggled(bool)'), self._c4_toggle)
        gui.connect(self._c5, SIGNAL('toggled(bool)'), self._c5_toggle)
        gui.connect(self._c6, SIGNAL('toggled(bool)'), self._c6_toggle)
        gui.connect(self._c7, SIGNAL('toggled(bool)'), self._c7_toggle)
        
        gui.connect(self._ok, SIGNAL('clicked()'), self._ok_clicked)
        gui.connect(self._cancel, SIGNAL('clicked()'), self._cancel_clicked)

        self._set_from_sep()
        self._gui.show()


    def _set_from_sep(self):
        data = self._sep.get_selection()
        row = filter(None, map(lambda i : data['row'][i] and self.row[i], range(len(self.row))))
        for f in row:
            f.toggle()
        col = filter(None, map(lambda i : data['col'][i] and self.col[i], range(len(self.col))))
        for g in col:
            g.toggle()
        custom_row = self._sep.get_custom_row()
        custom_col = self._sep.get_custom_col()
        if custom_row:
            self._r0.setText(custom_row)
        if custom_col:
            self._c0.setText(custom_col)
            
    def _cancel_clicked(self):
        self._sep.goto_default()

    def _ok_clicked(self):
        result = self._sep.get_symbols()

    def _c0_text(self, text):
        self._sep.set_custom_col(str(text))
        
    def _r0_text(self, text):
        self._sep.set_custom_row(str(text))

    def _r1_toggle(self, bool):
        if bool:
            self._sep.set_row_select('\t')
        if not bool:
            self._sep.set_row_select('\t', remove = True)
            
    def _r2_toggle(self, bool):
        if bool:
            self._sep.set_row_select(',')
        if not bool:
            self._sep.set_row_select(',', remove = True)

    def _r3_toggle(self, bool):
        if bool:
            self._sep.set_row_select(';')
        if not bool:
            self._sep.set_row_select(';', remove = True)

    def _r4_toggle(self, bool):
        if bool:
            self._sep.set_row_select('CuStOm')
        if not bool:
            self._sep.set_row_select('CuStOm', remove = True)
            self._sep.set_custom_row(None)
            self._r0.clear()

    def _r5_toggle(self, bool):
        if bool:
            self._sep.set_row_select(' ')
        if not bool:
            self._sep.set_row_select(' ', remove = True)

    def _r6_toggle(self, bool):
        if bool:
            self._sep.set_row_select(':')
        if not bool:
            self._sep.set_row_select(':', remove = True)

    def _r7_toggle(self, bool):
        if bool:
            self._sep.set_row_select('\n')
        if not bool:
            self._sep.set_row_select('\n', remove = True)

    def _c1_toggle(self, bool):
        if bool:
            self._sep.set_col_select('\t')
        if not bool:
            self._sep.set_col_select('\t', remove = True)

    def _c2_toggle(self, bool):
        if bool:
            self._sep.set_col_select(',')
        if not bool:
            self._sep.set_col_select(',', remove = True)

    def _c3_toggle(self, bool):
        if bool:
            self._sep.set_col_select(';')
        if not bool:
            self._sep.set_col_select(';', remove = True)

    def _c4_toggle(self, bool):
        if bool:
            self._sep.set_col_select('CuStOm')
        if not bool:
            self._sep.set_col_select('CuStOm', remove = True)
            self._sep.set_custom_col(None)
            self._c0.clear()

    def _c5_toggle(self, bool):
        if bool:
            self._sep.set_col_select(' ')
        if not bool:
            self._sep.set_col_select(' ', remove = True)

    def _c6_toggle(self, bool):
        if bool:
            self._sep.set_col_select(':')
        if not bool:
            self._sep.set_col_select(':', remove = True)

    def _c7_toggle(self, bool):
        if bool:
            self._sep.set_col_select('\n')
        if not bool:
            self._sep.set_col_select('\n', remove = True)

