# -*- coding: utf-8 -*-
# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.

from GUI.guinode import GUINode
from qtui import QWidgetFactory
from qt import *
import os
from Library.Util.Config import Config
from System.FileHandlers.Output.FileWriter import FileWriter
from GUI.NodeConfig import NodeConfig
from qt import QObject, QInputDialog

connect = QObject.connect

class OutputGUINode(GUINode):
    def __init__(self, parent, node):
        self._node = node
        GUINode.__init__(self, parent, node)
        self.changeIcon()
        self._output_node_dialog = OutputNodeDialog(node, self, parent)

    def nodeDialog(self):
        self._output_node_dialog.show()

    def changeIcon(self):
        nodename = self._node.getFileWriterClassName()
        nodeConfig = NodeConfig()
        nodeConfig.readConfig(nodename)
        imageURL = (nodeConfig.getImage()) + ".png"
        
        if imageURL.find("Unknown") > 0:
            nodeConfig.readConfig("OutputNode")
            imageURL = (nodeConfig.getImage()) + ".png"
        
        self.getCanvasNode().setImage(imageURL) 


class OutputNodeDialog:
    """Show settings for output node."""
    def __init__(self, node, gui_node, parent):
        self._node = node
        self._gui_node = gui_node
         
        self.file_name_parameter = node.getParameterByName("Filename")
        self.file_type_parameter = node.getParameterByName("Filetype")
        
        factory = QWidgetFactory()
        self._widget = factory.create(os.path.join(Config.uidir, "multipleoutputnodewidget.ui"), None, parent, None)
        self._ready_output_node_gui()

    def show(self):
        self.clear_fields()
        self._load_saved_values()
        self._widget.show()

    def addPorts(self):
        self._list_box_input_ports.clear()
        for port in self._node.getInputPorts():
            name = port.getName()
            type = port.getDataType()
            desc = port.getDescription()
            self._list_box_input_ports.insertItem(PortItem(name, type, desc))
    
    def _load_saved_values(self):
        self.addPorts()
        self._add_port_que = {}
        self._delete_port_que = []
        self._file_path = self.file_name_parameter.getValue()
        self._file_type = self.file_type_parameter.getValue()
        self._line_edit_filetype.setCurrentItem(self._file_type_list.index(self._file_type))
        self._text_edit_filetype_description.setText(self._node.getFileTypeDescription(self._file_type))
        self._line_edit_filename.setText(self._file_path)

    def _ready_output_node_gui(self):
        """Ready inputnode dialog with functionality that cannot be
        added in the ui-file.
        """

        dialog = self._widget

        #Getting children
        self._push_button_add = dialog.child("push_button_input_port_add")
        self._push_button_update = dialog.child("push_button_input_port_update")
        self._push_button_remove = dialog.child("push_button_input_port_remove")
        self._line_edit_portname = dialog.child("line_edit_input_port_name")
        self._line_edit_porttype = dialog.child("line_edit_input_port_type")
        self._text_edit_portdescription = dialog.child("text_edit_input_port_description")
        self._line_edit_filename = dialog.child("line_edit_filename")
        self._line_edit_filetype = dialog.child("combo_box_filetype")
        self._push_button_browse = dialog.child("push_button_browse")
        self._push_button_ok = dialog.child("push_button_ok")
        self._push_button_cancel = dialog.child("push_button_cancel")
        self._text_edit_filetype_description = dialog.child("text_edit_filetype_description")
        self._list_box_input_ports = dialog.child("list_box_input_ports")

        #Connecting Signals on a widget to Slots
        dialog.connect(self._push_button_add, SIGNAL("clicked()"), self.slot_port_add)
        dialog.connect(self._push_button_update, SIGNAL("clicked()"), self.slot_port_update)
        dialog.connect(self._push_button_remove, SIGNAL("clicked()"), self.slot_port_remove)
        dialog.connect(self._push_button_browse, SIGNAL("clicked()"), self.slot_browse)
        dialog.connect(self._push_button_ok, SIGNAL("clicked()"), self.slot_ok)
        dialog.connect(self._push_button_cancel, SIGNAL("clicked()"), self.slot_cancel)
        dialog.connect(self._line_edit_filetype, SIGNAL("activated(int)"), self.slot_filetype_changed)
        dialog.connect(self._line_edit_filename, SIGNAL("textChanged(const QString &)"), self.slot_filename_changed)
        dialog.connect(self._list_box_input_ports, SIGNAL("highlighted(QListBoxItem *)"), self.slot_port_selected)

        #Insert list of file types
        self._file_type_list = self._node.getFileTypeList()
        self._line_edit_filetype.insertStrList(self._file_type_list)

        self._text_edit_filetype_description.setText(self._node.description)

    def slot_port_add(self):
        portname = str(self._line_edit_portname.text())
        if portname:
            porttype = str(self._line_edit_porttype.text())
            portdescription = str(self._text_edit_portdescription.text())
            port = PortItem(portname, porttype, portdescription)
            self._list_box_input_ports.insertItem(port)
            self._add_port_que[portname] = (porttype, portdescription)
            
            self.clear_fields()
            
    def slot_port_update(self):
        selected_index = self._list_box_input_ports.currentItem()
        selected_port = self._list_box_input_ports.item(selected_index)
        if selected_port:
            old_portname = selected_port.get_port_info()[0]
            portname = str(self._line_edit_portname.text())
            porttype = str(self._line_edit_porttype.text())
            portdescription = str(self._text_edit_portdescription.text())
        
            if portname and portname != old_portname:
                port = PortItem(portname, porttype, portdescription)
                self._list_box_input_ports.changeItem(port, selected_index)
                self._add_port_que[portname] = (porttype, portdescription)
                if old_portname in self._add_port_que.keys():
                    del self._add_port_que[old_portname]

                self._delete_port_que.append(old_portname)


    def slot_port_selected(self, port_item):
        if port_item:
            (name, type, description) = port_item.get_port_info()
            self._line_edit_portname.setText(name)
            self._line_edit_porttype.setText(type)
            self._text_edit_portdescription.setText(description)

    def slot_port_remove(self):
        selected_index = self._list_box_input_ports.currentItem()
        selected_port = self._list_box_input_ports.item(selected_index)
        if selected_port:
            portname = selected_port.get_port_info()[0]
            self._list_box_input_ports.removeItem(selected_index)
            self._delete_port_que.append(portname)
            if portname in self._add_port_que.keys():
                del self._add_port_que[portname]

            self.clear_fields()
        
    def slot_filetype_changed(self, index):
        self._file_type = self._line_edit_filetype.text(index)
        self._text_edit_filetype_description.setText(self._node.getFileTypeDescription(self._file_type))

    def slot_filename_changed(self, filename):
        if filename:
            self._file_path = str(filename)
            filetype = self._node.getCompatibleFileType(self._file_path)
            index = self._file_type_list.index(filetype)
            self._line_edit_filetype.setCurrentItem(index)
            self.slot_filetype_changed(index)
    
    def slot_browse(self):
        file_filters = self._node.getFilters()
        output_file_path = QFileDialog.getSaveFileName(Config.homedir, file_filters, self._widget, \
                "save_file_dialog", "Choose a file to save...")
        
        if output_file_path:
            self._file_path = str(output_file_path)
            self._line_edit_filename.setText(self._file_path)      

    def slot_ok(self):
        self.commit_changes()

    def close(self):
        self._widget.close()

    def slot_cancel(self):
        self.close()

    def commit_changes(self):
        self.file_name_parameter.setValue(self._file_path)
        self.file_type_parameter.setValue(self._file_type)
        self._node.setFileType(self._file_type)
        self._gui_node.changeIcon()
        
        for portname in self._add_port_que.keys():
            type = self._add_port_que[portname][0]
            desc = self._add_port_que[portname][1]
            self._node.add_input_port(portname, type, desc)

        for portname in self._delete_port_que:
            port = self._node.getInputPortByName(portname)
            if port:
                port.remove()

        self.close()

    def clear_fields(self):
        self._line_edit_portname.setText(None)
        self._line_edit_porttype.setText(None)
        self._text_edit_portdescription.setText(None)
        self._line_edit_portname.setFocus()


class PortItem(QListBoxText):
    def __init__(self, port_name, port_type, port_description):
        QListBoxText.__init__(self, port_name)
        self._port_name = port_name
        self._port_type = port_type
        self._port_description = port_description

    def get_port_info(self):
        return (self._port_name, self._port_type, self._port_description)


class FileWriterGUINode(GUINode):
    """CanvasView-represenation of FileWriterNode."""
    def __init__(self, parent, node):
        GUINode.__init__(self, parent, node)
        self._writers = []
        self._data_names = {}
        # keep connection between list view items and data sources
        self._item_ports = {}
        self._update_icon_to_writer()

        self._parent = parent

        # widgets
        self._filename = None
        self._file_type_list = None
        self._storage_list = None
        self._widget = None

    def _query_rename(self, dummy=None):
        """Show dialog to user to rename a data source entry."""
        item = self._storage_list.selectedItem()
        if not item:
            return
        name = self._data_names[item]
        output_port = self._item_ports[item]
        
        text, changed = QInputDialog.getText("Renaming %s"%name ,
                                             "Enter new name for data:",
                                             QLineEdit.Normal, name,
                                             self._widget)
        if changed:
            self._data_names[item] = str(text)
            item.setText(0, text)

    def _update_widgets(self):
        self._widget = QWidgetFactory.create(os.path.join(Config.uidir,
                                                          "filewriternode.ui"),
                                             None, self._parent)        

        self._filename = self._widget.child('filename')
        self._file_type_list = self._widget.child('filetype_list')
        self._storage_list = self._widget.child('storage_list')
        browse_button = self._widget.child('browse_button')
        cancel_button = self._widget.child('cancel_button')
        ok_button = self._widget.child('ok_button')
        rename_button = self._widget.child('rename_button')

        connect(browse_button, SIGNAL('clicked()'), self._browse)
        connect(cancel_button, SIGNAL('clicked()'), self._widget.close)
        connect(ok_button, SIGNAL('clicked()'), self._ok)
        connect(rename_button, SIGNAL('clicked()'), self._query_rename)
        connect(self._storage_list, SIGNAL('doubleClicked(QListViewItem*)'),
                self._query_rename)
        connect(self._storage_list, SIGNAL('spacePressed(QListViewItem*)'),
                self._query_rename)
        connect(self._filename, SIGNAL("textChanged(const QString &)"),
                self._filename_changed)

        for writer in self.node.get_file_writers():
            self._file_type_list.insertItem(writer.get_description())
            # keep score of which writer is which in list
            self._writers.append(writer.get_filetype())

        # fill with info from node
        self._filename.setText(self.node.get_filename())

        # select correct value in combobox
        try:
            index = self._writers.index(self.node.get_file_writer())
        except ValueError:
            index = 0
        self._file_type_list.setCurrentItem(index)

        # fill data source info from connected ports
        self._storage_list.clear()
        self._data_names = {}
        self._item_ports = {}
        for name, port in self.node.get_storage_names():
            item = QListViewItem(self._storage_list, name, port.getName())
            self._item_ports[item] = port.getDataSource()
            self._data_names[item] = name
        
    def nodeDialog(self):
        self._update_widgets()
        self._widget.exec_loop()
        
    def _browse(self):
        output_path = QFileDialog.getSaveFileName("", self.node.get_filters(),
                                                  self._widget,
                                                  "file_writer_browser",
                                                  "Choose a file to save...")
        if output_path:
            self._filename.setText(output_path)
                                                  
    def _rename_source(self, list_view_item, new_name):
        self._data_names[list_view_item] = new_name
        list_view_item.setText(0, new_name)

    def _ok(self):
        self.node.set_filename(str(self._filename.text()))
        self.node.set_file_writer(self._writers[self._file_type_list.currentItem()])
        for item, name in self._data_names.items():
            port = self._item_ports[item]
            self.node.set_stored_name(port, name)
        self._widget.close()
        self._update_icon_to_writer()

    def _filename_changed(self, new_name):
        """Allow extension of filename to adjust writer used."""
        path, extension = os.path.splitext(str(new_name))
        # remove . from extension
        extension = extension[1:]
        extension_table = self.node.get_writers_by_extension()
        if extension in extension_table:
            writer = extension_table[extension]
            index = self._writers.index(writer.get_filetype())
            self._file_type_list.setCurrentItem(index)

    def _update_icon_to_writer(self):
        nodeConfig = NodeConfig()
        nodeConfig.readConfig(self.node.get_file_writer_class())
        imageURL = (nodeConfig.getImage()) + ".png"
        
        if imageURL.find("Unknown") > 0:
            nodeConfig.readConfig("FileWriterNode")
            imageURL = (nodeConfig.getImage()) + ".png"
        
        self.getCanvasNode().setImage(imageURL) 
