# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.



from dprint import *
from GUI.Plots import Filter
from GUI.Plots.SimplePlotDataFilter import SimplePlotDataFilter

class PlotDataFilter(SimplePlotDataFilter):

    def __init__(self, data = None, plot = None):
        SimplePlotDataFilter.__init__(self, data)

        self.plot = plot
        
        self.range = ["*"]
        
        # Just some good starting values
        self.globalMin = 1000000000
        self.globalMax = - 1000000000

    def getPlotData(self):
        """

        Excepton:
                 - IndexError - If the data set is composed of data sources with varying dimensions.
        """
        # if self.data:
        #     return self.data
        
        result = []

        if self.range[0] == "*":
            self.range = self.computeFullRange()
        
        # Go through all the lines that should be plotted
        for line in self.range:
            # Store the partial result (the resulting row at the end)
            partresult = []
       
            # Go through all filters and fetch the columns/rows they define
            for filter in self.filters:
                tempdata = [[]]
                try:                    
                    tempdata = self.data[filter.variableName]
                    
                except KeyError, e:
                    dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: Could not get data from variable: " +
                           filter.variableName + ". Error: " + str(e) + ". Maybe the port is new and has no data yet?")
                    

                if filter.direction == "column":
                    
                    # Go through all columns/rows                        
                    for col in filter.elements:
                        # The user starts from 1, we start from 0!
                        col = int(col) - 1
                        if len(tempdata) < int(line) + 1:
                            # print "col: Halloooooebn"
                            raise IndexError, ("Creating data failed!\n" +
                                               filter.variableName +
                                               " is reported to have " + str(len(tempdata)) +
                                               " rows. Tried to add a row number " +
                                               str(int(line) + 1) + ".\n"+
                                               "Check that the data range is not set too wide.\n" +
                                               "Also, if you mix rows and colums in a data set,\n" +
                                               "know what you are doing!")

                        if len(tempdata[int(line)]) < int(col) + 1:
                            # print "Driiiit"
                            raise IndexError, ("Creating data failed!\n" +
                                               filter.variableName + " has only " +
                                               str(len(tempdata[int(line)])) +
                                               " columns. Tried to add data from column "
                                               + str(int(col) + 1) + ".\n"+
                                               "Check that the data range is not set too wide.\n" +
                                               "Also, if you mix rows and colums in a data set,\n" +
                                               "know what you are doing!")
                        
                        partnumber = tempdata[int(line)][int(col)]

                        # Find maximum and minimum
                        if partnumber > self.globalMax:
                            self.globalMax = partnumber
                        if partnumber < self.globalMin:
                            self.globalMin = partnumber
                            
                        partresult.append(partnumber)

                elif filter.direction == "row":

                    for row in filter.elements:

                        row = int(row) - 1
                        
                        if len(tempdata) < int(row) + 1:
                            raise IndexError, ("Creating data failed!\n" +
                                               filter.variableName +
                                               " is reported to have " +
                                               str(len(tempdata)) +
                                               " rows. Tried to add a row number " +
                                               str(int(row) + 1) + ".\n" +
                                               "Check that the data range is not set too wide.\n" +
                                               "Also, if you mix rows and colums in a data set,\n" +
                                               "know what you are doing!")
                                                   

                        if len(tempdata[int(row)]) < int(line) + 1:
                            # print "PlotDataFilter: Hoooo"
                            raise IndexError, ("Creating data failed!\n" +
                                               filter.variableName + " has only " +
                                               str(len(tempdata[int(line)])) +
                                               " columns. Tried to add data from column " +
                                               str(int(line) + 1) + ".\n"+
                                               "Check that the data range is not set too wide.\n" +
                                               "Also, if you mix rows and colums in a data set,\n" +
                                               "know what you are doing!")
                        
                        partnumber = tempdata[int(row)][int(line)]

                        if partnumber > self.globalMax:
                            self.globalMax = partnumber
                        if partnumber < self.globalMin:
                            self.globalMin = partnumber
                            
                        partresult.append(partnumber)
            
            result.append(partresult)

        return result

    def getMinMax(self):
        dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: returning min: ", self.globalMin, " and max: ", self.globalMax)
        return [self.globalMin, self.globalMax]

    def addFilter(self, variablename, direction, elements):
        self.data = {}
        self.filters.append(Filter.Filter(variablename, direction, elements))

    def setRange(self, range):
        # self.data = {}
        self.range = range
        dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: Setting range: " + str(self.range))

    def updateRangeToData(self):
        oldrange = self.range
        self.range = self.computeFullRange()
        if oldrange[0] == "*":
            oldrange = self.range
        
        datlength = len(self.getPlotData())

        length = oldrange[-1]

        while length >= datlength and length > 0:
            dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: updating range to data. Removing element value " + str(length))
            length = oldrange[-2]
            del oldrange[-1]

        self.range = oldrange

        dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: Updating range to match data. New range is:\n" +
               str(self.range))

    def computeFullRange(self):
        """Find out what a full range is :)"""

        resList = []
        filt = None
        try:            
            filt = self.filters[0]
        except IndexError, e:
            dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: Can not compute full range!" +
                   "There are no filters set for the plot.")
            return []

        dat = [[]]
        try:
            dat = self.data[filt.variableName]
        except:
            dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: Can't compute full range, maybe data has not come to the input port?")

        if filt.direction == "row":
            for elem in xrange(0, len(dat[0])):
                resList.append(elem)

        else:
            for elem in xrange(0, len(dat)):
                resList.append(elem)
        dprint(flags.GUI.PLOTS.SETUP, "PlotDataFilter: Have computed a full range:\n" + str(resList))

        return resList
        

    def deleteLines(self, lines):

        # Range translation from the plot data set to the
        # original data set
        rangetrans = []
        for line in lines:
            rangetrans.append(self.range[line])

        print "Range to be removed:"
        print rangetrans

    def hasVariable(self, variable):
        for filter in self.filters:

            if filter.variableName == variable:
                return 1
        return 0

    def getRealIndex(self, index):
        """Takes an index seen from the plot and trasnlates it to the real
        index of the data set."""

        try:
            realIndex = self.range[index]
            return realIndex
        except IndexError:
            return -1

    def getRemoveStatement(self, plotIndex):
        realIndex = self.getRealIndex(plotIndex)
        retList = []
        for filter in self.filters:

            if type(filter) == str:
                dprint(flags.GUI.PLOTS.RUN,
                       "PlotDataFilter: Trying to add remove data for a plot with a simple data filter. This operation is not permitted (well actually it is not implemented)")
            
            else:
                if filter.direction == "row":
                    for name in self.filterControl.controller.getSetupTab().getPortAliases(filter.variableName):
                        retList.append(["removeColumns", {"data" : [name], "columns" : [realIndex]}])
                        
                elif filter.direction == "column":
                    for name in self.filterControl.controller.getSetupTab().getPortAliases(filter.variableName):
                        retList.append(["removeRows", {"data": [name], "rows" : [realIndex]}])

                else:
                    dprint(flags.GUI.PLOTS.RUN,
                           "PlotDataFilter: Cannot determine direction of filter. MAJOR BUG!")
            
        return retList

    def getPlotIndex(self, pindex):

        try:
            plotIndex = self.range.index(pindex)
            return plotIndex
        
        except ValueError:
            return -1
        
    def pickedEvent(self, index, range):
        dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: picked event")
        if len(range) == len(self.computeFullRange()):
            dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: range and fullrange is both " , len(range))
            realIndex = self.getPlotIndex(index)
            dprint(flags.GUI.PLOTS.RUN,  "PlotDataFilter: Real index is " , realIndex)
            
            if not realIndex == -1:
                dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: Actor within range to be picked...")
                self.plot.pickActor(realIndex)
        
    def unPickedEvent(self, index, range):
        dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: unpicked event")
        if len(range) == len(self.computeFullRange()):
            dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: range and fullrange is both " , len(range))
            realIndex = self.getPlotIndex(index)
            dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: Real index is " , realIndex)
            
            if not realIndex == -1:
                dprint(flags.GUI.PLOTS.RUN, "PlotDataFilter: Actor within range to be unpicked...")
                self.plot.unPickActor(realIndex)

                
                
                
        
