#-*- coding: utf-8 -*-

import qt
import os
import sys
from System.Data import Data
from System.Data.Node import Node
from Library.Util.gui import TreeNode
from Library.Util.Config import Config
from xml.dom import minidom

# Temporarily, until æøå is properly handled in XML
import base64

# Øystein TODO: Clean up execute()

class SelectionCollection(dict):
    """A collection of selections
        key: name, value: [index2d]
    """

    def __init__(self):
        """Load the selections from file."""
        self.filename = os.path.join(Config.userconfigdir, "rfilter")
        self.clear()
        try:
            data = open(self.filename).read()    
            names, selections = self.from_XML(data)
            self.update(dict(zip(names, selections)))
        except IOError:
            self.clear()

    def save(self): 
        """Save the selections to file."""
        file = open(self.filename, "w")
        s = self.to_XML()
        file.write(str(s))
        file.close()

    def to_XML(self):
        """Generate XML, return a string."""
        s = u'<?xml version="1.0" encoding="utf-8"?>\n'
        s += "<filter>\n"
        for name, selection in self.items():
            s += '\t<selection name="%s">\n' % base64.encodestring(name)
            for f in selection:
                s += '\t\t<element data="%s" />\n' % str(f)
            s += '\t</selection>\n' 
        s += '</filter>\n'
        return s
            
    def from_XML(self, data):
        """Parse XML, return a list of names and a list of selections."""
        document = minidom.parseString(data)
        tags = document.getElementsByTagName("selection")
        names = []
        selections = []
        for tag in tags:
            names.append(base64.decodestring(str(tag.attributes["name"].value)))
            elements = tag.getElementsByTagName("element") 
            tmplist = []
            for element in elements:
                numlist = str(element.attributes["data"].value)[1:-1].split(",")
                tmplist.append(tuple([int(siffer) for siffer in numlist if siffer]))
            selections.append(tmplist)
        return (names, selections)

#    def __setitem__(self, name, selection):
#        """Add a name and a connected selection."""
#        key = name
#        while key in self:
#            key += "_"
#        dict.__setitem__(self, key, selection)


class RDissectionNode(Node):

    def __init__(self, name):
        Node.__init__(self, name)
        self.setCanAddInputPorts(True)
        self.setCanAddOutputPorts(True)
        self.description = "R data field picker node"
        self.add_input_port("RObject", "", "")
        self._forest = []
        self.selection = []
        self.selection_names = None
        self.selectioncollection = SelectionCollection()
        self.output = []
        
    def get_forest(self):
        """Returns a forest of TreeNodes."""
        return self._forest

    def get_selection(self):
        """Returns the active selection."""
        return self.selection 

    def get_selection_name(self, selection):
        """Returns a name of a given selection."""
        names = self.selectioncollection.values()
        if selection in names:
            index = names.index(selection)
            return self.selectioncollection.keys()[index]
        else:
            return False

    def get_selectioncollection(self):
        """Returns the list of saved selections."""
        return self.selectioncollection

    def _genXML(self):
        """Generates XML of active selection."""
        s = '\t<rdissection>\n'
        for item in self.output:
            s += "\t\t" + '<item index="%s" meta="' % str(item.index2d)
            if "matrix" in item.info:
                s += "matrix"
            elif "scalar" in item.info:
                s += "scalar"
            s += '" name="%s" />\n' % item.name
        s += '\t</rdissection>\n'
        return s
        
    def toXML(self):
        """Inserts xml of selections into Node's XML 
           (called from Node upon save)"""
        self.nodeRepName
        xml_list = Node.toXML(self)
        xml_swap = xml_list[:-1]
        xml_swap.extend(self._genXML().split('\n'))
        xml_swap.append(xml_list[-1])
        return xml_swap
            
    def from_xml(self, selection_list):
        """Sett selection from XMLReader"""
        self.selection = dict(selection_list).keys()
        self.selection_names = dict(selection_list).values()
    
    def ok_from_gui(self, active_selection):
        """This one is called from the GUI-twin-node."""
        name, selection = active_selection
        self.selection = selection
        self.output_from_selection()
        self.remove_old_ports()
    
    def save_from_gui(self, active_selection):
        """Saves filter pr. user.
            active_selection is (name, [index2d, ...])
        """
        name, selection = active_selection
        # name could be a QString, hence the str
        self.selectioncollection[str(name)] = selection
        self.selectioncollection.save()

    def load_from_gui(self, selection_name):
        """Loads a filter.
            Returns the new active selection.
            The format is still (name, [index2d, ...])
        """
        return (selection_name, self.selectioncollection[selection_name])

    def remove_from_gui(self, selection_name):
        """Removes a filter."""
        del self.selectioncollection[selection_name]
        
    def output_from_selection(self):
        self.output = []
        for path in self.selection:
            this_node = self._forest[path[0]]
            for element in path[1:]:
                this_node = this_node.children[element]
            self.output.append(this_node)
        for treenode in self.output:
            name, info, data = treenode.name, treenode.info, treenode.data
            self.add_output_port(name, "", info)
            port = self.getNodeOutputPort(name)
            port.setData(Data.Data(name, data))
                
    def remove_old_ports(self):
        portlist = self.getOutputPorts()[:]
        names = map(lambda element: repr(element), self.output)
        for port in portlist:
            if port.getName() not in names:
                self.remove_port(port)

    
    def execute(self):
        # Take all the input data and make Trees of them. A Tree consists of TreeNodes.
        # A forest is naturally a collection of trees.
        self._forest = [Tree(data,i).get_root() for i, data in enumerate([port.getData() for port in self.inPorts])]
        # Show the Trees to the user
        #self._postEvent("INFO", "\n".join(map(str, forest)))
            
        if self.selection:
            self.output_from_selection()
            self.remove_old_ports()

        Node.execute(self)

class Tree:

    def __init__(self, data, index):
        """Build a tree from the given data."""
        self.root = self.get_children_of(data, data.getName(), [index])

    def get_children_of(self, data, name, index2d=[]):
        """Creates a TreeNode from data recursively.
            @param data A System.Data.Data.Data instance
            @param name The TreeNode-name
            @return A TreeNode that contains branches for all the given data.
        """
        meta_special = data.getMetadataSpecial()
        generic, special = meta_special['generic'], meta_special['special']
        this_node = TreeNode(name, data.getData())
        num_children = 0
        # Handle the three R-variations
        if "names" in generic:
            if special.has_key('fields'):
                names = generic["names"].getData()
                for child_index, child_name in enumerate(names):
                    child_data = special["fields"][child_index]
                    # Creates TreeNodes for every child to the given data and name
                    this_node.add_child(self.get_children_of(child_data, child_name, index2d + [child_index]))
                num_children += child_index + 1
        elif "row.names" in generic:
            if special.has_key('fields'):
                sf = special["fields"]
                for i, (key, value) in enumerate(sf.items()):
                    this_node.add_child(self.get_children_of(value, key, index2d + [i]))   
                num_children += i + 1
        elif "dimnames" in generic:
            this_node.add_child(self.get_children_of(generic["dimnames"], name + "$dimnames", index2d + [0]))
            num_children += 1
        keylist = generic.keys()
        keylist.sort()
        for key in keylist:
            if key not in ["dimnames", "row.names", "names", "class", "dim"]:
                if generic[key].getData() or (len(generic[key].getMetadataSpecial()['generic']) > 1):
                    this_node.add_child(self.get_children_of(generic[key], key, index2d + [num_children]))
                    num_children = num_children + 1
                
        
        this_node.set_info()
        this_node.index2d = tuple(index2d)
        return this_node

    def get_root(self):
        """Get the root of the tree."""
        return self.root


