# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.

from System.Data.Node import Node
from System.Data.Data import Data

class StructDissectionNode(Node):

    def __init__(self, name):
        Node.__init__(self, name)
        self._outputs = {}
        self._input_port = self.add_multi_input_port('struct input')

    def get_all_struct_data(self):
        structs = []
        for port in self.get_all_input_ports():
            if port.getData():
                data = port.getData()
                if isinstance(data.getData(), dict):
                    structs.append(data)

        return structs

    def set_output(self, name, sub_elements = []):
        if not sub_elements:
            port = self.add_output_port(name)
        else:
            port = self.add_output_port("%s-%s" % (name, '-'.join(sub_elements)))

        self._outputs[port] = (name, sub_elements)

    def execute(self):
        for port, (name, elements) in self._outputs.items():
            self.set_output_data(port, name, elements[:])

    def set_output_data(self, port, name, sub_elements):
        structs = self.get_all_struct_data()
        return_data = None

        for input_struct in structs:
            if input_struct.getName() == name:
                return_data = input_struct.getData()
                break

        if not return_data:
            return

        data_name = name
        sub_elements.reverse()        
        while sub_elements:
            element = sub_elements.pop()
            return_data = return_data[element]
            data_name = element
            
        port.setData(Data(data_name, return_data))
        
    def clear_output(self):
        for port in self.getOutputPorts():
            self.remove_port(port)

        self._outputs = {}

    def remove_output(self, removable):
        list_ = removable.split('-')
        list_.reverse()

        name = list_.pop()
                    
        for port, outputs in self._outputs.items():
            if (name, list_) == outputs:
                self.remove_port(port)
                break

    def add_output_port(self, variable_name, *args):
        
        port = Node.add_output_port(self, variable_name, *args)

        list_ = variable_name.split('-')
        name = list_[0]
        list_.remove(name)

        self._outputs[port] = (name, list_)
        
        return port
