# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.

import types
import copy
import traceback

from System.Data import Node
from System.Data import Parameter
from System.Data.Data import Data
from Library.Nodes.FileHandlers.OutputNode import OutputNode
from System.FileHandlers.Output.MoleculeFileWriter import MoleculeFileWriter

class FragmentOutputNode(OutputNode):

    def __init__(self, name):
        Node.Node.__init__(self,name)
        
        self.setCanAddInputPorts(True)
        self.setCanAddOutputPorts(False)

        self.description = "Node for writing Molecule-files to disk for later use as fragments."
        self.writer = MoleculeFileWriter()
        self.add_input_port("molecule","-", "-")
        self.addParameter("dummyIndex", "Integer", "Attachpoint - 1-indexed", 1, None, None, None, 1, None)


    def execute(self):
        # set dummy index on molecule

        for param in self.getParameters():
            if param.getName() == "dummyIndex":
                dummyIndex = param.getValue()

                for port in self.inPorts:
                    if port.getName() == "molecule":
                        portData = port.getData()
                        model = portData.getData()[0]
                        modelcopy = copy.deepcopy(model)
                        modelcopy.dummyIndex = dummyIndex - 1 # 1-indexed to 0-indexed
                        port.setData(Data(portData.getName(), [modelcopy]))

        OutputNode.execute(self)
    
#       """Automagically adds a parameter object for each """
#     def add_output_port(self, variableName, validType, description,options="NA"):
#         number = str(len(self.getParameters()))       
#         if options != "FORCE":
            
#             self.addParameter("Output nr " + str(number),"Integer","Value of Output " + str(number),default = 1)

#       return Node.Node.add_output_port(self, variableName, validType, description,"FORCE")

