# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.

import os
import random
import re
import types
import sys
import Numeric

from dprint import *
from Library.Plugins.RPluginFiles import S4Object
from Library.Util import Config
from System import Tools
from System.Exceptions.ZPluginError import *
from System.Data.Data import Data

from Library.Util.rtester import RTester, RInstallError

try:
    RTester.r_install_test()
    import rpy
except RInstallError:
    pass
    
    
#TODO: More persistent in removing temporary R-variables. If attach fails, temporary variables are laying all
#      over the place.
#TODO: Find out what should be done by R-scripters, and what should be done by SciCraft. No need to reimplement
#      R in SciCraft..
#TODO: All types of datasetting needs to be able to deal with data-objects which has no metadata. In fact, the
#      only data-objects with the correct metadata are the ones made by this class in the first place. However, 
#      data-objects originate from several other places too.
#      DONE FOR METADATA. Still have to fix this for the getTypeSpecial() calls (adapt __getDefaultType()?)
#TODO: Convert S4Object to data-objects. No need for this one exception. An alternative solution is for S4Object to
#      subclass Data. This solution shoud be considered. - DONE BUT NOT TESTED THOROUGLY -
#TODO: setList() and setMatrix() have to be able to work without metadata. Should be straight-forward. Tip: setMatrix-indata should be 
#      python-arrays. If not, try casting (Numeric.array(mylist)).
#      FINISHED? (check this!)
#TODO: Write code comments.
#TODO: In Windows: NAN appers as 1.#QNAN. Must be replaced with something... See RPlugin for inspiration.
#TODO: Extensive testing.
# Sudden segmentation faults could be:
#       * Unicode passed to R. R (or RPy) does definitely not like unicode.
#       * Using NA with factors when setting variables (python --> R)
#       * Generally unstable RPy-behavior
#
# Changelog:
# 29. february - changed the way varpaths are handled due to problems with escaping and splitting of varpaths.
#                Lists are now passed around instead of strings.
# 17. february - removed creation of environment-file from getEnvironment. Not supporting custom environments. Using globalenv
#                when rebuilding.
# 14. february - Fixed bug in __getAccessibleVardesc(). Failed when input was a named list with other lists
#                due to matching criteria (e.g in 'myList$output[[12]]', 'output[[12]]' does not match 'output').
# 14. february - Added support for new R classes. New get methods (there are corresponding set methods):
#                getCall(), getEnvironment(), getSymbol(), getTerms().
#                [NOTE] get-/setEnvironment are NOT finished!
# 14. february - Fixed bug in getVector(). Behaved incorrectly when getting named vectors. Also updated setVector().
#  8. february - Added support for R 'array'.
#  7. february - Had a look at ordering of attributes/slots in S4 objects. Seems to be a little more complex than first expected.
#                Simply setting them in the correct order is not enough, as we firste instanciate an object and them set the
#                attributes/slotnames. For example, 'data(swirl)' and 'new("marrayRaw")' create S4 objects that are structurally
#                equal, except for the ordering. 'swirl' is of class 'marrayRaw', do not know why the ordering differ.
#                Although the objects fails the 'identical()' test, we get a fuzzy match ('all.equal()') and the ordering does not
#                seem to have any practical importance.
#                [NOTE] As a consequence of this, 'testRObjectHandler' is rewritten to accept fuzzy matches.
#  7. february - partially rewrote setMatrix() and cleaned up code. Fixed bug due to introduction of Numeric.array in Data-objects.
#                Empty matrix represented by "zeros((0,), 'l')", missed by if test for empty matrix.
#  6. february - rewrote __getDefaultType(). It now returns a dictionary with four keys; 'class', 'is', 'mode' and 'typeof'.
#  4. february - setGeneralVars implemented for setting variables with no need for metadata. Now uses setGenericMeta which was missing from
#                earlier implementation.
#  4. february - extended __setRVar to handle data-objects without R-specific metadata for reconstruction.
#  3. february - getMatrix now produces Numeric.arrays
#  3. february - all rpy.r.assigns sent to assignRVar() for assigning and error-catching.
#  3. february - convertLongList() implemented for recursing into lists and casting longs to floats.



class RObjectHandler:

    def __init__(self):

        # Flags used in dprint:
        self.DEBUG          = 0
        self.TRACE          = 0
        self.DEVINFO        = 0
        self.SHOWMETA       = 0
        self.SHOWTYPEDEFS   = 0

        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__init__(1)#entry]")
        self.specialAttr = 'special' # String for metadata-dictionary. Specific information needed to rebuild object in R.
        self.genericAttr = 'generic' # String for metadata-dictionary. General information set generically.
        self.NA          = -2147483648 # This value is returned by RPy when dealing with a R-NA-'object' 

        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__init__(1)#exit]")
        
        
    def getType(self, var):
        """Returns the R output from the commands mode, typeof, is and class
            as a dictionary.

            Params:
                var - String : The R name of the variable/object to execute the commands on.

            Returns:
                A dictionary with four keys: "mode", "typeof", "class" and "is".
                If 'var' is unknown or otherwise invalid, 'None' is inserted as values.
        """

        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getType(2)#entry]")
        if type(var) == types.ListType:
            (var, ignore) = self.__getAccessibleVardescr(var)
        res = {}
        res['mode']   = rpy.r('mode('+var+')')
        res['class']  = rpy.r('class('+var+')')
        res['is']     = rpy.r('is('+var+')')
        res['typeof'] = rpy.r('typeof('+var+')')
        dprint(self.SHOWTYPEDEFS, "  [getType(2)] Typedefs for object '" + str(var) + "':")
        dprint(self.SHOWTYPEDEFS, "               class=", res['class'], "/ is="    , res['is'])
        dprint(self.SHOWTYPEDEFS, "               mode=" , res['mode'] , "/ typeof=", res['typeof'])
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getType(2)#exit]")
        return res
       



    def getS4Obj(self, var): # NB! Parents and children are not being set now. Not that it is used anywhere now,
                             # but it might be necessary when making S4Obj-editnode(s)
                             # Should we run __getRVarValue just in case? 
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getS4Obj(2)#entry]")
        if (type(var) is not types.StringType):
            raise AttributeError, "Wanted string, got something completely different"
            
        if not self.isS4Obj(var): 
            raise ZPluginError, "Rootnode %s not an S4Object" % var

        node = S4Object.S4Object(self.__getFriendlyName(var), self.__getRVarValue(var)) # Now using __getRVarValue
        node.setTypeSpecial(self.getType(var))
        # Do not include full R var path, only last name.
        typ = self.getType(var)
        meta = {}
        meta[self.genericAttr] = self.__fetchGenericMeta(var)
        meta[self.specialAttr] = {}
        meta[self.specialAttr]['class'] = typ['class']
        node.setMetadataSpecial(meta)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getS4Obj(2)#exit]")
        return node


    
    def getRVar(self, var, name=None, varTyp=None):
        """
            Fetches any supported object from R. The method contains logic
            for deciding what class the spcified object has, and then using
            the correct get-method to fetch the object. 

            WARNING: Special care must be taken if the get-methods are called
                     directly, as they do not currently have enough sanity
                     checks to handle invalid data (data of wrong class).

            Params:
                var     - string        : The name of the R variable/object to
                                          fetch.
                name    - string        : The name to be stored in the Data
                                          object. If not specified, the last
                                          element of the object path given in
                                          'var' is used.
                varTyp  - dictionary    : Workaround. Currently only used when
                                          handling 'AsIs' objects. This tells
                                          the method to use the provided type
                                          definition (type override).
            Returns:
                A Data object, with a representation of the fetched R object.
                The Data object normally contains metadata needed to
                assemble the object in R at a later time. If the data is
                edited, the metadata might have to be edited to reflect the
                changes as well. Note that the data accessed by Data.getData()
                is meant to be a simple Python representation of the R data,
                and that more data may be stored in the meta section.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getRVar(2/4)#entry]")
        dprint(self.DEBUG, "  [getRVar(2/4)] Getting var:", var)
        
        if name   == None: name = self.__getFriendlyName(var)
        # [WARNING] Very hacky! Might work though :)
        if type(var) is not types.ListType:
            var = [var]
        # [NOTE] Do NOT use accVar in getType! This breaks things, as getType() runs __getAccessibleVardesc() iself.
        if varTyp == None: varTyp = self.getType(var)
        dprint(self.DEBUG, "  [getRVar(2/4)] Typedefinition:" , varTyp)
        # Note that we overwrite the var variable here. This is intentional.
        (accVar, var) = self.__getAccessibleVardescr(var)
        length = rpy.r('length(' + accVar + ')')
        data = None
        isTyp = Tools.assertList(varTyp['is'])
        if self.isS4Obj(var):
            data = self.getS4Obj(var)

        elif varTyp['class'] == 'environment':
            data = self.getEnvironment(var, name)

        elif varTyp['class'] == 'name':
            data = self.getSymbol(var, name)
            
        elif varTyp['class'] == 'array':
            data = self.getArray(var, name)

        elif varTyp['class'] == 'matrix':
            data = self.getMatrix(var, name)
        
        elif varTyp['class'] == 'data.frame':
            data = self.getFrame(var, name)

        elif varTyp['class'] == 'factor':
            data = self.getFactor(var, name)

        elif 'call' in isTyp and 'language' in isTyp:
            data = self.getCall(var, name)

        elif 'terms' in isTyp and varTyp['typeof'] == 'language':
            data = self.getTerms(var, name)

        elif varTyp['class'] == 'AsIs' or 'AsIs' in Tools.assertList(varTyp['class']):
            data = self.getAsIs(var, name)  

        elif varTyp['class'] == 'list' or (varTyp['mode'] == 'list' and varTyp['typeof'] == 'list'):
            data = self.getList(var, name)

        elif 'vector' in isTyp and length > 1:
            data = self.getVector(var, name)

        elif (varTyp['class'] == 'numeric'   or
              varTyp['class'] == 'character' or
              varTyp['class'] == 'complex'   or
              varTyp['class'] == 'integer'   or
              varTyp['class'] == 'logical'   or
              varTyp['class'] == 'NULL'):

            meta = {self.genericAttr: self.__fetchGenericMeta(var), self.specialAttr: {}}
            meta[self.specialAttr]["class"] = varTyp['class']
            if varTyp["class"] != 'NULL':
                meta[self.specialAttr]["hasNA"] = 0    
            res = self.__getRVarValue(var)
            if res == self.NA: 
                if varTyp["class"] != 'NULL':
                    meta[self.specialAttr]["hasNA"] = 1                    
                res = [None] # DIRTY! #
            data = Data(name, res)
            data.setTypeSpecial(varTyp)
            data.setMetadataSpecial(meta)

        else:
            raise Exception, "Type not supported. Library.Plugins.RObj.py.getRVar(). Varname %s, vartype %s" % (var, varTyp)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getRVar(2/4)#exit]")

        return data

        

    def getArray( self, var, name ):
        """Retrieves R objects of class 'array'.
            
            Params:
                var  - string : The name of the R variable to fetch.
                name - string : The name to set on the Data object.
                
            Returns:
                A Data object holding a representation of the array object.
                The Data object only has one entry in the special metadata 
                section; 'class'.
        """
        # [NOTE] Currently, rpy fetches named arrays as a list. Support for named arrays must
        #        be implemented if rpy changes this to dictionary.
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getArray(3)#entry]")
        (accVar, var) = self.__getAccessibleVardescr(var)
        meta = {self.specialAttr: {}, self.genericAttr: self.__fetchGenericMeta(var)} 
        meta[self.specialAttr]['class'] = "array"
        data = Data(name, rpy.r(accVar))
        data.setTypeSpecial(self.getType(var))
        data.setMetadataSpecial(meta)        
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getArray(3)#exit]")
        return data
    

        
    # Somewhat temporary solution. Should be gone thoroughly through.
    def getAsIs(self, var, name):
        """
            Fetches R object classified as 'AsIs' objects. This is done by
            first removing the 'AsIs' entries in the class definition, and
            then running the getRVar() method again with the manipulated
            type definition. This way, the correct get-method will be used
            to fetch the object. Finally, the 'AsIs' entries will be readded.

            Params:
                var  - string : The name of the R variable/object to fetch.
                name - string : The name to be stored in the Data object.
                                If not specified, the last element of the 
                                variable path 'var' will be used.

            Returns:
                A Data object representing the 'AsIs' object. The contents of
                the meta section depends on the real object class. For details,
                please see the documentation for the appropriate get-methods.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getAsIs(3)#entry]")
        # Overwrite getType()["class"] and rerun getRVar().
        varTyp = self.getType(var)
        if len(Tools.assertList(varTyp["class"])) > 1:
            varTyp["class"] = varTyp["class"][-1]
        else:
            varTyp["class"] = varTyp["mode"]

        data = self.getRVar(var, varTyp = varTyp)
        varTyp = data.getTypeSpecial()["class"]
        varTyp = Tools.assertList(varTyp)
        if "AsIs" not in varTyp:
            varTyp.insert(0, "AsIs")
            data.getTypeSpecial()['class'] = varTyp
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getAsIs(3)#exit]")
        return data
    
        
    
    def getCall( self, var, name ):
        """
            Fetches a R call. A call is an expression that can be executed.
            
            Params:
                var  - string : The name of the R call object to be fetched.
                name - string : The names to be stored in the Data object.
                                If not specified, the last element of the
                                variable path 'var' will be used.

            Returns:
                A Data object representing a R call. This Data object stores 
                the call as a list in the special metadata section under the
                key 'callList'. Furthermore, a textual representation meant
                for human interpretation is stored in the object's data 
                varaiable.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getCall(3)#entry]")
        tmpVar = self.getUniqueVarname()
        (accVar, var) = self.__getAccessibleVardescr(var)
        rpy.r(tmpVar + '<- as.list(' + accVar + ')')
        listData = self.getList(tmpVar, "callList")
        meta = {self.specialAttr: {}, self.genericAttr: self.__fetchGenericMeta(var)}
        meta[self.specialAttr]['class']    = "call"
        meta[self.specialAttr]['callList'] = listData
        data = Data(name, rpy.r('format(' + accVar + ')'))
        data.setTypeSpecial(self.getType(var))
        data.setMetadataSpecial(meta)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getCall(3)#exit]")
        return data

        
    def getEnvironment( self, var, name ):
        """
            This method simply stores a text string indicating that an 
            environment was located in the object. We are do not want to
            store the actual environment the user had when creating the R
            object we are fetching.

            Params:
                var : ListType : A list describing the R path to the
                  ; Environment object.
                name : StringType : The name of the object (from R).

            Returns:
                A Data object.
            """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getEnvironment(3)#entry]")
        meta = {self.specialAttr: {'class': 'environment'}, self.genericAttr: self.__fetchGenericMeta(var)}
        data = Data(name, ".Environment")
        data.setTypeSpecial(self.getType(var))
        data.setMetadataSpecial(meta)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getEnvironment(3)#exit]")
        return data 


    def getFactor(self, var, name):
        """
            Fetches a R factor and stores it in a Python representation. 
            
            Params:
                var  - string : The name of the R variable/object to be fetched.
                name - string : The names to be stored in the Data object.
                                If not specified, the last element of the
                                variable path 'var' will be used.
            
            Returns:
                A Data object representing a R factor. The Data object has two
                keys in the special metadata section; 'as.integer' and
                'levels'. The former is all the integer data values in the
                factor, the latter is the labels for the integer values.
                The data variable in the Data object contains has a human
                readable representation of the factor, where the integer values
                have been replaced by the corresponding label.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getFactor(3)#entry]")
        (accVar, var) = self.__getAccessibleVardescr(var)
        typeDef = self.getType(var)
        meta_special = {}
        meta_special["as.integer"] = rpy.r("as.integer(" + accVar + ")") # Does one always have to look for "$" to get this information?
        meta_special["levels"]     = rpy.r("levels(" + accVar + ")")
        meta_special["class"]      = typeDef["class"]
        meta_generic = self.__fetchGenericMeta(var, meta_special)
        meta = {self.genericAttr: meta_generic, self.specialAttr: meta_special}

        data = Data(name, self.__getRVarValue(var))
        data.setTypeSpecial(typeDef)
        data.setMetadataSpecial(meta)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getFactor(3)#exit]")
        return data
        


    def getFrame(self, var, name):
        """
            Fetches a R data.frame object. These objects are collections of
            tightly coupled variables, and resembles matrices.
            
            Params:
                var  - string : The name of the R variable/object to be fetched.
                name - string : The names to be stored in the Data object.
                                If not specified, the last element of the
                                variable path 'var' will be used.
            
            Returns:
                A Data object representing the data frame. The Data object has
                two keys in the special metadata section; 'names' and 'fields'.
                The former lists the names of the variables in the data frame,
                if any names are set. The latter contains Data objects for each
                varaible. These objects can represent many R classes, depending
                on the type of data in the variables.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getFrame(3)#entry]")
        (accVar, var) = self.__getAccessibleVardescr(var)
        typ  = self.getType(var)
        meta = {self.specialAttr: {}, self.genericAttr: {}}
        meta[self.specialAttr]["class"] = typ['class']
        fields = {}
        names  = Tools.assertList(rpy.r('names('+accVar+')'))
        for nam in names:
            # Must make copy here.
            newVarPath = var[0:]
            newVarPath.append(nam)
##            fields[nam] = self.getRVar(var + "$" + nam, nam)
            fields[nam] = self.getRVar(newVarPath, nam)
                    
        meta[self.specialAttr]['names']  = names
        meta[self.specialAttr]['fields'] = fields
        meta[self.genericAttr] = self.__fetchGenericMeta(var, meta[self.specialAttr])
        dat = Data(name, self.__getRVarValue(var))#var.split("$")[-1], rpy.r(var))
        dat.setTypeSpecial(typ)
        dat.setMetadataSpecial(meta)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getFrame(3)#exit")
        return dat



    def getList(self, var, name):
        """
            Fetches a R list.
            
            Params:
                var  - string : The name of the R variable/object to be fetched.
                name - string : The names to be stored in the Data object.
                                If not specified, the last element of the
                                variable path 'var' will be used.

            Returns:
                A Data object representing the R list. The object has a key
                'fields', which is a list with one element for each element
                in the R list. This is neccessary as R lists can contain
                any R object, and the type of one element is independent of
                the types of the other elements (as opposed to for instance
                R matrices).
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getList(3)#entry]")
        meta = {self.specialAttr: {}, self.genericAttr: self.__fetchGenericMeta(var)}
        meta[self.specialAttr]['class']  = 'list'
        meta[self.specialAttr]['fields'] = []
        (accVar, var) = self.__getAccessibleVardescr(var)
    

        # RPy return float for int value. Convert.
        length = int(rpy.r('length('+ accVar + ')'))
        # Some objects cheat on their length...
        if length != 0:
            try:
                rpy.r(accVar + "[[" + str(length) + "]]")
            except Exception, e:
                length = int(rpy.r('length(names(' + accVar + '))'))
                dprint(self.DEBUG, "Invalid list size detected. Fetched alternative!")
        for i in xrange(length):
            # [NOTE] Because the varpath has been "optimized", we must pass on the string here.
            meta[self.specialAttr]['fields'].append(self.getRVar(accVar + "[[" + str(i + 1) + "]]"))
        data = Data(name, self.__getRVarValue(var))
        data.setTypeSpecial(self.getType(var))
        data.setMetadataSpecial(meta)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getList(3)#exit]")
        return data
        
        

    def getMatrix(self, var, name):
        """Text
            Params:
                    var - type : expl

            Returns:
                    var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getMatrix(3)#entry]")
        (accVar, var) = self.__getAccessibleVardescr(var) 
        varTyp = self.getType(var)
        meta   = {self.specialAttr: {'class': varTyp['class']}, self.genericAttr: self.__fetchGenericMeta(var)}
        #res = Numeric.array(self.__getRVarValue(var))
        res = self.__getRVarValue(var)
        if res == None: res = [] # RPy returns None for empty matrices
        dat = Data(name, res)
        dat.setTypeSpecial(varTyp)
        dat.setMetadataSpecial(meta)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getMatrix(3)#exit]")
        return dat
       
        

    def getSymbol( self, var, name ):
        """
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getSymbol(3)#entry]")
        meta = {self.specialAttr: {'class': 'name'}, self.genericAttr: self.__fetchGenericMeta(var)}
        (accVar, var) = self.__getAccessibleVardescr(var)
        data = Data(name, rpy.r('as.character(' + accVar + ')'))
        data.setMetadataSpecial(meta)
        data.setTypeSpecial(self.getType(var))
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getSymbol(3)#exit]")
        return data
        

    
    def getTerms( self, var, name ):
        """
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getTerms(3)#entry]")
        (accVar, var) = self.__getAccessibleVardescr(var)
        meta = {self.specialAttr: {'class': ['terms', 'formula']}, self.genericAttr: self.__fetchGenericMeta(var)}
        tmpName = self.getUniqueVarname()
        rpy.r(tmpName + ' <- format(' + accVar + ')')
        data = Data(name, rpy.r(tmpName))
        data.setMetadataSpecial(meta)
        data.setTypeSpecial(self.getType(var))
        
#        data = self.getRVar(tmpName, name)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getTerms(3)#exit]")
        return data
        

    
    def getVector(self, var, name):
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getVector(3)#entry]")
        (accVar, var) = self.__getAccessibleVardescr(var)
        varTyp = self.getType(var)
        classT = varTyp['class']
        src = self.__getRVarValue(var)
        src = Tools.assertList(src)
        res = []
        meta                   = {}
        meta[self.specialAttr] = {}
        meta[self.genericAttr] = self.__fetchGenericMeta(var)
        meta[self.specialAttr]['class'] = classT

        if (classT == 'numeric'   or
            classT == 'character' or
            classT == 'complex'   or
            classT == 'integer'   or 
            classT == 'logical'):

            meta[self.specialAttr]["hasNA"] = 0
            
            for i in src:
                if i == self.NA: 
                    res.append(None)
                    meta[self.specialAttr]["hasNA"] = 1
                else: res.append(i)

        
        elif classT == 'list':
            # Should actually never happen, as list-vectors are indeed lists...
            raise Exception, "Got R class 'list' where it should not occur. Programming error!"
        else:
            raise Exception, "Unsupported vector-type. Vector-type: %s. Please report this incident." % classT

        if meta[self.genericAttr].has_key('names'):
            tmpNames = self.getUniqueVarname()
            rpy.r(tmpNames + ' <- names(' + accVar + ')')
            rpy.r('names(' + accVar + ') <- NULL')
            dataNoNames = rpy.r(accVar)
            rpy.r('names(' + accVar + ') <- ' + tmpNames)
            rpy.r.rm(tmpNames)
            meta[self.specialAttr]['dataNoNames'] = dataNoNames
        dat = Data(name, res)
        dat.setTypeSpecial(varTyp)
        dat.setMetadataSpecial(meta)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.getVector(3)#exit]")
        return dat
    
    def setRVar( self, rvar, pathList = [], name = None, use_unique = False):
        # TODO: If indata has no metadata, and indata is a dictionary, let rpy convert the whole thing.
        # This method shoud also distribute metadataless objects by first assigning necessary meta.
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setRVar(2/3)#entry]")
        # Try, if rvar if invalid, let __setRVar raise the exceptions...


        if use_unique:
            # use unique name
            name = self.getUniqueVarname()
        elif not name:
            # extract the name from the data object
            name = rvar.getName()



        res = self.__setRVar(rvar, pathList)
        try:
            self.__attach(res, pathList, name)
        except:
            print "Warning. Could not attach", name, res.getName()
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setRVar(2/3)#exit]")
        return name

    def setS4Obj(self, s4obj, pathList = []):
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setS4Obj(2)#entry]")
        # Make sure we have an instance of S4Object.
        if not isinstance(s4obj, S4Object.S4Object):
            raise TypeError, "[setS4Obj] setS4Obj must be called with an instance of S4Object."
        # Create en "empty" instance of the S4 object.
        tmpName = self.getUniqueVarname()
        try:
            s4Class = s4obj.getTypeSpecial()['class']
            rpy.r(s4obj.getName() + ' <- new("' + s4Class + '")')
        except KeyError:
            raise ZPluginError, "[setS4Obj] No class specified for s4 object. Unable to continue."
        except Exception, e:
            dprint(self.DEBUG, " " , e)
            raise ZPluginError, "[setS4Obj] Unable to instanciate S4 class '" + str(s4Class) + "'."
        newPathList = pathList[0:]
        newPathList.append(s4obj.getName())
        genMeta = s4obj.getMetadataSpecial()[self.genericAttr]
        for dictKey in genMeta:
            el = genMeta[dictKey]
            if isinstance(el, S4Object.S4Object):
                self.__attach(self.setS4Obj(el, []), newPathList, el.getName())
            else:
                self.__attach(self.__setRVar(el, newPathList), newPathList, el.getName())
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setS4Obj(2)#exit]")

        return s4obj.getName()


    
    def setArray( self, array, pathList = [], name = None ):
        """
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setArray(2)#entry]")
        metaSpecial = self.__setDefaultMeta(array, "array")
        typeSpecial = self.__assertDictionary(array.getTypeSpecial())
        tmpName = self.getUniqueVarname()
        aMode = None
        if typeSpecial.has_key("mode"):
            aMode = typeSpecial["mode"]
        rpy.r.assign(tmpName, array.getData())
        rpy.r(tmpName + ' <- array(' + tmpName + ')')
        if aMode != None:
            rpy.r('mode(' + tmpName + ') <- "' + aMode + '"')
        self.setGenericMeta(tmpName, metaSpecial[self.genericAttr])

        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setArray(2)#exit]")
        return tmpName
        
        

    def setAsIs( self, asIs, pathList = [], name = None ):
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setAsIs(2)#entry]")
        # Currently not needed.
        # metaSpecial = self.__setDefaultMeta(asIs, "AsIs")
        tmpName = self.getUniqueVarname()
        typeSpecial = self.__assertDictionary(asIs.getTypeSpecial())
        # See if there are any other classes in "class". If not, use mode to take action.
        if typeSpecial.has_key('class'):
            classDef = Tools.assertList(typeSpecial['class'])
            typeDef = typeSpecial.copy()
            if len(classDef) > 1:
                # Remove "AsIs", this is ASSUMED to be at position 0. Is this always true?
                typeDef['class'] = typeDef['class'][1:]
            else:
                if typeDef.has_key('mode'):
                    typeDef['class'] = typeDef['mode']
                else:
                    typeDef = self.__getDefaultType(asIs.getData())
        else:
            typeDef = self.__getDefaultType(asIs.getData())
        typeDef['is'] = []
        asIs.setTypeSpecial(typeDef)
        rVarName = self.__setRVar(asIs, [])
        asIs.setTypeSpecial(typeSpecial)
        rpy.r(tmpName + ' <- I(' + rVarName + ')')
        rpy.r('rm(' + rVarName + ')')

        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setAsIs(2)#exit]")
        return tmpName


        
    def setCall( self, call, pathList = [], name = None ):
        """
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setCall(2/4)#entry]")
        metaSpecial = self.__setDefaultMeta(call, "call")
        tmpName = self.setList(metaSpecial[self.specialAttr]['callList'])
        rpy.r(tmpName + ' <- as.call(' + tmpName + ')')
        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setCall(2/4)#exit]")
        return tmpName


        
    def setEnvironment( self, environment, pathList = [], name = None ):
        """
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setEnvironment(2/4)#entry]")
        tmpName = self.getUniqueVarname()
        rpy.r(tmpName + ' <- globalenv()')
        
        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setEnvironment(2/4)#exit]")
        return tmpName
        
        

    def setFactor(self, factor, pathList = [], name = None):
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setFactor(2/4)#entry]")
        metaSpecial = self.__setDefaultMeta(factor, "factor")
        fData = metaSpecial[self.specialAttr]["as.integer"]
        if fData == []:
            # An empty factor requires special care.
            tmpName   = self.getUniqueVarname()
            rpy.r(tmpName + ' <- vector(mode="numeric", length=0)')
            tmpLevels = self.getUniqueVarname()
            rpy.r(tmpLevels + ' <- vector(mode="character", length=0)')
        else:
            tmpName   = self.getUniqueVarname()
            self.assignVar(tmpName, fData)
            tmpLevels = self.getUniqueVarname()
            self.assignVar(tmpLevels, metaSpecial[self.specialAttr]["levels"])
        rpy.r(tmpName + ' <- factor(' + tmpName + ', labels=' + tmpLevels + ')')
        rpy.r('rm(' + tmpLevels + ')')
        self.setGenericMeta(tmpName, metaSpecial[self.genericAttr])

        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setFactor(2/4)#exit]")
        return tmpName



    def setFrame(self, frame, pathList = [], name = None):
        """
            Rebuilds a 'data.frame' in the R environment.

            Params:
                frame - System.Data.Data : A Data object created by getFrame().
                  ; This object contains the information needed to rebuild
                  ; the 'data.frame' in R.

            Returns:
              The variable name of the 'data.frame' built in R.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setFrame(2)#entry]")
        metaSpecial = self.__setDefaultMeta(frame, "data.frame")
        tmpName = self.getUniqueVarname()
        tmpVars = []
        addStrings = []
        convertToChr = []
        fields = metaSpecial[self.specialAttr]["fields"]
        # Make sure frames are added in correct order.
        for frameName in metaSpecial[self.specialAttr]["names"]:
            tmpVars.append(self.__setRVar(fields[frameName]))
            # We do not want to convert character vectors to factors.
            if rpy.r('class(' + tmpVars[-1] + ')') == "character":
                addStrings.append("I(" + tmpVars[-1] + ")")
                convertToChr.append(frameName)
            else:
                addStrings.append(tmpVars[-1])
        rpy.r(tmpName + ' <- data.frame(' + ','.join(addStrings) + ')')
        
        # Overwrite the 'names' attribute to remove the tmp varnames.
        tmpVar  = self.getUniqueVarname()
        self.assignVar(tmpVar, metaSpecial[self.specialAttr]["names"])
        rpy.r('names(' + tmpName + ') <- ' + tmpVar)
        
        # Convert selected character vectors marked 'AsIs' back to character vectors.
        for frame in convertToChr:
            rpy.r('class(' + tmpName + '$' + frame + ') <- "character"')
        
        # Clean up...
        tmpVars.append(tmpVar)
        rpy.r('rm(' + ','.join(tmpVars) + ')')
        
        # Set generic meta that might have been present in the frame.
        self.setGenericMeta(tmpName, metaSpecial[self.genericAttr])

        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setFrame(2)#exit]")
        return tmpName



    def setList(self, list, pathList = [], name = None):
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setList(2/4)#entry]")
        metaSpecial = self.__setDefaultMeta(list, "list")
        tmpName = self.getUniqueVarname()
        rpy.r(tmpName + ' <- list()')
        tmpVar = self.getUniqueVarname()
        # [NOTE] Workaround for lists containing NULL. See last part in end of method (setList()).
        magicIndex = len(metaSpecial[self.specialAttr]["fields"]) +1
        rpy.r(tmpName + '[[' + str(magicIndex) + ']] <- "ERRANDOUS ENTRY!"')
        index = 1
        for field in metaSpecial[self.specialAttr]["fields"]:
            tmpVar = self.__setRVar(field, [])
            # Do not assign NULL to list, as this deletes the list element...
            if rpy.r('class(' + tmpVar + ')') != 'NULL':
                rpy.r(tmpName + '[[' + str(index) + ']] <- ' + tmpVar)
            rpy.r('rm(' + tmpVar + ')')
            index += 1

        # [NOTE] Workaround for lists containg NULL. See first part further up.
        rpy.r(tmpName + '[[' + str(magicIndex) + ']] <- NULL')
        self.setGenericMeta(tmpName, metaSpecial[self.genericAttr])

        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setList(2/4)#exit]")
        return tmpName



    def setMatrix(self, matrix, pathList = [], name = None):
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setMatrix(2)#entry]")
        dprint(self.SHOWMETA, "  [setMatrix(2/3)] Matrix metadata:" , matrix.getMetadataSpecial())
        metaSpecial = self.__setDefaultMeta(matrix, "matrix")
        typeSpecial = self.__assertDictionary(matrix.getTypeSpecial())
        tmpName = self.getUniqueVarname()
        tmpVar = self.getUniqueVarname()
        if typeSpecial.has_key("typeof") and typeSpecial["typeof"] != 'set_value_here!':
            mMode  = ',mode="' + typeSpecial["typeof"] + '"'
            mModeR = typeSpecial["typeof"]
        else:
            mMode  = ''
            mModeR = "any"
            dprint(self.DEVINFO, "WARNING: No mode set for matrix/vector in RObjectHandler.setMatrix().")
        mData = matrix.getData()
        # Need try - except here to handle cases where data is 'None' or other unsized object.
        try:
            length = len(mData)
        except:
            length = 0
            dprint(self.DEVINFO, "WARNING: Data in matrix was unsized object.")
        if length == 0:
            # Deal with empty matrix.
            dprint(self.DEBUG, "  [setMatrix(2/3)] Creating empty matrix.")
            rpy.r(tmpName + ' <- matrix(data=vector(length=0'+ mMode + '), nrow=0, ncol=0)')
            self.setGenericMeta(tmpName, metaSpecial[self.genericAttr])
        else:
            dprint(self.DEBUG, "  [setMatrix(2/3)] Creating matrix with data.")
            rows = 1 
            if type(mData[0]) is types.ListType or \
                type(mData[0]) is Numeric.ArrayType:
                rows = len(mData)
            self.assignVar(tmpName, mData)
            byrow = "FALSE"
            if type(mData) is not Numeric.ArrayType:
                rpy.r(tmpName + ' <- unlist(' + tmpName + ')')
                byrow = "TRUE"
            # Coerce to original type if known.
            if rpy.r('typeof(' + tmpName + ')') != mModeR and mModeR != "any":
                if byrow == "FALSE":
                    byrow = "TRUE"
                    rpy.r(tmpName + ' <- unlist(' + tmpName + ')')
                rpy.r(tmpName + ' <- as.vector(' + tmpName + ', mode="' + mModeR + '")')
            rpy.r(tmpName + ' <- matrix(data=' + tmpName + ', nrow=' + str(rows) + ', byrow=' + byrow + ')')
            self.setGenericMeta(tmpName, metaSpecial[self.genericAttr])

        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setMatrix(2)#exit]")
        return tmpName
    
    
    
    def setSymbol( self, symbol, pathList = [], name = None ):
        """
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setSymbol(2/4)#entry]")
        # Don't need meta here.
        tmpName = self.getUniqueVarname()
        rpy.r.assign(tmpName, symbol.getData())
        rpy.r(tmpName + ' <- as.symbol(' + tmpName + ')')
#        self.__setGenericMeta()
        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setSymbol(2/4)#exit]")
        return tmpName



    def setTerms( self, terms, pathList = [], name = None ):
        """
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setTerms(3)#entry]")
        metaSpecial = self.__setDefaultMeta(terms)
        tmpName = self.getUniqueVarname()
        rpy.r.assign(tmpName, terms.getData())
        rpy.r(tmpName + ' <- terms.formula(as.formula(' + tmpName + '))')
        self.setGenericMeta(tmpName, metaSpecial[self.genericAttr])
    
        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setTerms(3)#exit]")
        return tmpName
        
    

    def setVector(self, vector, pathList = [], name = None):
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setVector(2/4)#entry]")
        metaSpecial = self.__setDefaultMeta(vector, "vector")
        tmpName = self.getUniqueVarname()
        if metaSpecial[self.specialAttr].has_key('dataNoNames'):
            vData = metaSpecial[self.specialAttr]['dataNoNames']
        else:
            vData = vector.getData()
        if vector.getTypeSpecial().has_key("typeof"):
            vMode = ',mode="' + vector.getTypeSpecial()["typeof"] + '"'
        else:
            vMode = ""
            dprint(self.DEVINFO, "WARNING: No mode set for vector in RObjectHandler.setVector().")
        if vData == [] or vData == None:
            rpy.r(tmpName + ' <- vector(length=0' + vMode + ')')
        else:
            if metaSpecial[self.specialAttr]["hasNA"] == 1:
                # We must change all 'None' to self.NA.
                dprint(self.DEVINFO, "WARNING: 'None' presence indicated in data for vector object '" + str(vector.getName()) + "', must replace with 'NA'.")
                # [NOTE] Should we make a copy?
                vData = Tools.assertList(vData)
                for i in xrange(len(vData)):
                    if vData[i] == None:
                        vData[i] = self.NA
            self.assignVar(tmpName, vData)
            rpy.r(tmpName + ' <- as.vector(' + tmpName + vMode + ')')
        self.setGenericMeta(tmpName, metaSpecial[self.genericAttr])
        # If method is called directly, not through getRVar(), or we want to rename the variable.
        if name != None:
            self.__attach(tmpName, pathList, name)
            tmpName = name
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setVector(2)#exit]")
        return tmpName

 
 
    def isS4Obj(self, slotsObj): # Hacky implementation. Better solution?
        """
            Params:
                - slotsObj : String - the object-name (full) at hand.
            Return:
                - int - 0 if object is not an s4-object, 1 otherwise.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.isS4Obj(2)#entry]")
        if self.getType(slotsObj)['mode'] != 'list': # Hacky
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.isS4Obj(2)#exit_01]")
            return 0
        try:
            slots = rpy.r('slotNames(' + slotsObj+ ')')
        except:
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.isS4Obj(2)#exit_02]")
            return 0 # should perhaps raise an exception
        if slots == None:  # Hacky
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.isS4Obj(2)#exit_04]")
            return 0
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.isS4Obj(2)#exit_04]")
        return 1

        

    def setGenericMeta( self, rvar, meta, pathList = [] ):
        """Text
                Params:
                        var - type : expl

                Returns:
                        var - type : expl
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setGenericMeta(3/4)#entry]")
        # What about ordering here? We could store the attibute order if necessary.
        for key in meta.keys():
            dprint(self.DEBUG, "  [setGenericMeta(3/4)] Setting attribute '" + str(key) + "'")
            varName = self.__setRVar(meta[key])
            self.__attach(varName, [rvar], key)

        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setGenericMeta(3/4)#exit]")



    def setGeneralVars(self, rvar):
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setGeneralVars(2)#entry]")
        # Use tmp name, as not all names in a data structure (names(X)) can be used a variable name.
        #data = rvar.getName()
        data = self.getUniqueVarname()
        self.assignVar(data, rvar.getData())
        genMeta = rvar.getMetadataSpecial()
        if type(genMeta) is types.DictType:
            if self.genericAttr in genMeta.keys():
                self.setGenericMeta(data, rvar.getMetadataSpecial()[self.genericAttr])
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.setGeneralVars(2)#exit]")
        return data

        
       
    def __setRVar(self, rvar, pathList = []): # Is Pathlist needed?
        """
            Sets the R object represented by 'rvar' in the current R 
            environment. This method does some preprocessing/validation, then
            it dispatches the appropriate method to handle the object.
            
            Params:
                rvar     - Data object : The Data object to be set in R.
                pathList - list        : An expression specifying the object/
                                         location of the R object. This can for
                                         instance be a slot in a S4 object.
                                         Default is '[]'.

            Returns:
                Returns the variable name (string) of the object set in R.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setRVar(2)#entry]")
        # Make sure we have an instance of a S4Object object or a Data object.
        if type(rvar) is not types.InstanceType:
            raise TypeError, "[__setRVar#001] Must be called with an instance as argument. Type of object received: " + str(type(rvar))
        data = None
        if isinstance(rvar, S4Object.S4Object):
            data = self.setS4Obj(rvar)
        elif isinstance(rvar, Data):
            if type(rvar.getTypeSpecial()) is not types.DictType:
                # [NOTE] If no type definition exists, create a default one instead of raising an exception. OK?
                dprint(self.DEVINFO, "WARNING: No type definition found for object '" + str(rvar.getName()) + "'. Creating a default one based on Python data type.")
                rvar.setTypeSpecial(self.__getDefaultType(rvar.getData()))
            typeSpecial = rvar.getTypeSpecial()
            classDef    = Tools.assertList(typeSpecial['class'])
            isDef       = []
            modeDef     = ""
            typeofDef   = ""
            if typeSpecial.has_key('is'):
                isDef = Tools.assertList(typeSpecial['is'])
            if typeSpecial.has_key('mode'):
                modeDef = typeSpecial['mode']
            if typeSpecial.has_key('typeof'):
                typeofDef = typeSpecial['typeof']
            dprint(self.DEBUG, "  [__setRVar(2/3)]" , rvar.getName() ,  "=" , classDef, "/" , isDef)
            # [NOTE] The test for vector must come AFTER the test for list.
            if "AsIs" in classDef or "AsIs" in isDef:
                data = self.setAsIs(rvar)

            elif classDef == ['call']:
                data = self.setCall(rvar)

            elif classDef == ['environment']:
                data = self.setEnvironment(rvar)

            elif classDef == ['name']:
                data = self.setSymbol(rvar)

            elif classDef == ['array']:
                data = self.setArray(rvar)
            
            elif classDef == ['factor']:
                data = self.setFactor(rvar)
            
            elif classDef == ['data.frame']:
                data = self.setFrame(rvar)
            
            elif classDef == ['matrix']:
                data = self.setMatrix(rvar)
            
            elif classDef == ['list'] or (modeDef == 'list' and typeofDef == 'list'):
                data = self.setList(rvar)
            
            elif 'vector' in isDef:
                data = self.setVector(rvar)

            elif 'terms' in isDef:
                data = self.setTerms(rvar)
            
            else:
                data = self.setGeneralVars(rvar)
        else:
            raise TypeError, "__setRVar must be called with a Data- or S4Object-instance as argument. Got instance of " + rvar.__module__
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setRVar(2)#exit]")
        return data
   

 
    def __attach( self, tmpName, pathList, realName = None ):
        """
            Moves a temporary variable to its real name and location.
            If 'pathList' is empty, the variable is in effect renamed.
            Otherwise, the temporary variable is inserted into another object.
            
            Params:
              tmpName  - StringType : The temporary variable name.
              pathList - ListType   : The path for the object the variable is to be inserted into.
              realName - StringType : The real variable name.

            Returns:
              Nothing.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__attach(3/4)#entry]")
        if realName == None:
            realName = tmpName
        if len(pathList) == 0:
            dprint(self.DEBUG, "  [__attach(3/4)] Move to realname:" , realName , "<-" , tmpName)
            rpy.r(realName + ' <- ' + tmpName)
            if realName != tmpName:
                rpy.r('rm(' + tmpName + ')')
        else:
            # [NOTE] This might be hairy...
            (accVar, ignore) = self.__getAccessibleVardescr(pathList)
            dprint(self.DEBUG, "  [__attach(3/4)] Attach expression: " + 'attr(' + accVar + ',"' + realName + '") <- ' + tmpName)
            rpy.r('attr(' + accVar + ',"' + realName + '") <- ' + tmpName)
            rpy.r('rm(' + tmpName + ')')
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__attach(3/4)#exit]")


 
    def __assertDictionary( self, obj ):
        """
            Takes a object as input, and returns it if it is a dictionary. 
            If not a dictionary, a empty dictionary is returned.

            Params:
              obj - object : The object to be verified.

            Returns:
              'obj' - DictType : if 'ojb' is a dictionary.
              '{}'  - DictType : if 'obj' is not a dictionary.
        """
        if type(obj) is types.DictType: return obj
        return {}
        

        
##    def __fetchGenericMeta(self, varPathStr, excludeDict={}):    
    def __fetchGenericMeta(self, varPath, excludeDict={}):
        """
            Stores any attributes that might have been set on R objects.
            
            Params:
              varPathStr  - StringType : A string representing the R variable expression for
                                         the object to store attributes for.
              excludeDict - DictType   : A dictionary whose keys specifies attributes to exclude.
                                         This is typically because some attributes require special care.

            Returns:
              A dictionary with a key for each attribute the object has,
              and the values are 'System.Data.Data' objects.
              If there are no attributes, a empty dictionary is returned.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__fetchGenericMeta(2/3)#entry]")
        (accVar, varPath) = self.__getAccessibleVardescr(varPath)
        names = Tools.assertList(rpy.r("names(attributes(" + accVar + "))"))
        meta = {}
        for name in names:
            if not excludeDict.has_key(name):
                # Make copy, or else we get some very bad variable names.
                newVarPath = varPath[0:]
                newVarPath.append(name)
                meta[name] = self.getRVar(newVarPath, name)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__fetchGenericMeta(2/3)#exit]")
        return meta


    def __handleIndexedVar( self, var ):
        # Indexed var must end with at least one ']'.
        if var[-1] != ']':
            return (var, "")
        pattern = "["
        if var[-2] == ']':
            # Assume dobble index.
            pattern = "[["
        indSt = var.rfind(pattern)
        if indSt == -1:
            return (var, "")
        return (var[0:indSt], var[indSt:])

    def __getAccessibleVardescr( self, varPath ):
        """
            Generates a string that accesses the object specified by 'varPath'
            in R. 
            
            Params:
                - varPath : listType
                ; 
            Returns:
                A tuple of length two; (accVar, varPath)
                NOTE: 'accVar' should be used in rpy calls. When passing the
                variable path to other methods, and the method used the
                __getAccessibleVardescr(), use 'varPath'.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getAccessibleVardescr(2)#entry]")
        inType = type(varPath)
        if inType is types.StringType:
            # When we handle list indexing, we send string. Do not warn about this. Relaxed checking...
            if varPath[-1] != ']' and ('$' in varPath or '@' in varPath or \
                '"' in varPath or "'" in varPath):
                dprint(self.DEVINFO, "  [__getAccessibleVardescr] WARNING: Received string. Passing on as is, unable to do anything.")
            return (varPath, [(1,varPath)])
        if inType is not types.ListType:
            raise ZPluginError, "Must be list. Got: " + str(inType)
        accVar = varPath[0]
        # Extract first element if it is a tuple (ie. it has been processed before).
        if type(accVar) is types.TupleType:
            accVar = accVar[1]
        if len(varPath) == 1:
            if type(accVar) is not types.StringType:
                raise ZPluginError, "ERROR!  Must have string in varpath. Got: " + str(type(accVar))
            return (accVar, [(1,accVar)])
        elEsc = None
        for element in varPath[1:]:
            if type(element) is types.TupleType:
                if element[0] == 1:
                    elEsc = element[1]
                element = element[1]
            else:     
                elEsc = re.escape(element)
            if elEsc != element:
                # Must handle indices explicitly.
                (varPart, indexPart) = self.__handleIndexedVar(element)
                tmp = varPart.replace('"', '\\"').replace("'", "\\'")
                elEsc = '"' + tmp + '"' + indexPart
            # Current algoritm is quite greedy, a lot of R calls are executed.
            # Can do this only once for every element?
            try:
                names      = Tools.assertList(rpy.r('names(' + accVar + ')'))
                attributes = Tools.assertList(rpy.r('names(attributes(' + accVar + '))'))
            except rpy.RExceptition:
                print "PROGRAMMING ERROR in RObjectHandler.getAccessibleVardescr()! PLEASE NOTIFY DEVELOPERS!"
                names      = []
                attributes = []
            if element in names:
                accVar += '$' + elEsc
            elif element in attributes:
                accVar = 'attributes(' + accVar + ')$' + elEsc
            else:
                # If it gets this far, we probably end up with a NULL, but what else to do?
                accVar += '$' + elEsc
                dprint(self.DEVINFO, "WARNING: Probably using invalid R varpath:", accVar) 
            
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getAccessibleVardescr(2)#exit_03]")
        return (accVar, [(1, accVar)])


    def getUniqueVarname(self):
        """
            Generates a unique variable name in the current R environment.

            Returns:
              A string representing a variable name.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getUniqueVarname(1)#entry]")
        current = rpy.r.ls()
        if type(current) is not types.ListType: current = [current]

        while 1:
            tmpname = "SciCraftTmp" + str(random.randint(1,10000000))
            if tmpname not in current: break

        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getUniqueVarname(1)#exit]")
        return tmpname


    def __getRVarValue(self, var):
        """
            Gets the data stored in the R variable specified. The result of this operation
            depends on RPy's abilities to convert from R datastructures to Python datastructures.

            Params:
              var - StringType : A variable name in R.

            Returns:
              A built-in Python object with the data from the R variable. Note that some information
              may be lost.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getRVarValue(2)#entry]")
        (accVar, varPath) = self.__getAccessibleVardescr(var)
        tmpRes = rpy.r(accVar)
        varType = self.getType(accVar)
        # We skip checking for Robj in matrices (cannot contain them).
        if varType['class'] == "matrix":
            res = tmpRes
        else:
            res = self.__replaceRobjInstances(tmpRes, varPath)
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getRVarValue(2)#exit]")
        return res


    def __convertToString(self, structure, varPath):
        resString = ""
        (accVar, ignore) = self.__getAccessibleVardescr(varPath)
        varType = self.getType(accVar)
        # We skip checking for Robj in matrices (cannot contain them).
        classSkipList = ["integer", "character", "logical", "numeric", 
            "complex", "NULL", "factor"]
        if varType['class'] == "matrix":
            return ""
        if varType['class'] in classSkipList:
            return ""
        if hasattr(structure, "__iter__"):
                # We can have either a dictionary or a list. RPy does not use tuples (yet?).
                if type(structure) == types.DictType:
                    for key in structure.keys():
                        newVarPath = varPath[:]
                        if key == "":
                            # Does this assumption hold?
                            # Problem occures when 'as.list' is called on a
                            # call object.
                            newVarPath[-1] = (1,
                                newVarPath[-1][1] + "[[1]]")
                        else:
                            newVarPath.append(key)
                        resString += self.__convertToString(structure[key], newVarPath)
                    return resString
                elif type(structure) == types.ListType:
                    rlength = int(rpy.r("length(" + accVar + ")"))
                    if rlength == 1 and len(structure) != rlength:
                        ignore.append("[[1]]")
                        return resString + self.__convertToString(structure, ignore)
                    for index in range(0, len(structure)):
                        #newVarPath = varPath[:]
                        newVarPath = ignore[:]
                        # Replace tuple in varPath to reflect the list index.
                        # Need this to be able to access the variable in R.
                        # [NOTE] First R index is 1, not 0.
                        
                        newVarPath[-1] = (1,
                            newVarPath[-1][1] + "[[" + str(index +1) + "]]")
                        resString += self.__convertToString(structure[index], newVarPath)
                    return resString

        else:
            return str(structure)



    def __replaceRobjInstances(self, structure, varPath):
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__replaceRobjInstances(3)#entry]")
        strStruct = self.__convertToString(structure, varPath)
        if strStruct.find("<Robj object at 0x") != -1:
            varType = self.getType(varPath)
            (accVar, ignore) = self.__getAccessibleVardescr(varPath)
            # Need to remove the Robj instances.
            if hasattr(structure, "__iter__"):
                # We can have either a dictionary or a list. RPy does not use tuples (yet?).
                if type(structure) == types.DictType:
                    for key in structure.keys():
                        newVarPath = varPath[:]
                        if key == "":
                            # Does this assumption hold?
                            # Problem occures when 'as.list' is called on a
                            # call object.
                            newVarPath[-1] = (1, 
                                newVarPath[-1][1] + "[[1]]") 
                        else:
                            newVarPath.append(key)
                        (accVar, ignore) = self.__getAccessibleVardescr(newVarPath)
                        componentType = self.getType(newVarPath)
                        (stopRecursion, result) = self.__getRobjReplacement(accVar, componentType)
                        if stopRecursion:
                            structure[key] = result
                        else:
                            structure[key] = self.__replaceRobjInstances(structure[key], newVarPath)
                    dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__" +
                        "replaceRobjInstances(3)#exit_01]")
                    return structure
                elif type(structure) == types.ListType:
                    for index in range(0, len(structure)):
                        newVarPath = varPath[:]
                        # Replace tuple in varPath to reflect the list index. 
                        # Need this to be able to access the variable in R.
                        # [NOTE] First R index is 1, not 0.
                        newVarPath[-1] = (1, 
                            newVarPath[-1][1] + "[[" + str(index +1) + "]]") 
                        (accVar, ignore) = self.__getAccessibleVardescr(newVarPath)
                        componentType = self.getType(newVarPath)
                        (stopRecursion, result) = self.__getRobjReplacement(accVar, componentType)
                        if stopRecursion:
                            structure[index] = result
                        else:
                            structure[index] = self.__replaceRobjInstances(structure[index], newVarPath)
                        #structure[index] = self.__replaceRobjInstances(structure[index], newVarPath)
                    dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__" +
                        "replaceRobjInstances(3)#exit_02]")
                    return structure
                else:
                    raise TypeError, "[RObjectHandler.__replaceRobjInstances] " +\
                        "Unsupported R data type representation " +\
                        "encountered. Please file a bug report! Unsupported" +\
                        " representation: " + str(type(structure))
            else:
                # We have a single value. Try to extract something meaningful...
                if rpy.r('is.atomic(' + accVar + ')'):
                    res = rpy.r('format(' + accVar + ')')
                else:
                    res = rpy.r('as.character(' + accVar + ')')
                dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__" +
                    "replaceRobjInstances(3)#exit_03]")
                return res
        
        # No Robj's found; return unaltered.
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__" +
            "replaceRobjInstances(3)#exit_04]")
        return structure
       
    
    def __getRobjReplacement(self, accVar, typeDef):
        """
            Fetches an alternative Python representation from R. This is
            normally done when RPy is unable to convert an object from R 
            itself. Rules match on the typedefinition of the R objects.
            Also note that this method is used to create an representation
            of an object on an earlier stage than it would have been if the
            recursion was fully completed. To see what this means, look at
            these two representations of a R call object:
              1) "randomForest(x = Classes ~ ., data = X, 
                    importance = TRUE, proximity = TRUE)"
              2) ["randomForest", "Classes ~ .", "X", "TRUE", "TRUE"]
            
            Obviously, number 1 gives more information to the user.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__" +
        "getRobjReplacement(3)#entry]")
        isType = Tools.assertList(typeDef['is'])
        classType = Tools.assertList(typeDef['class'])
        # Check if we should stop recursion here. This is done to improve 
        # readability for specific data types.
        if 'call' in isType and 'language' in isType and 'call' in classType:
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__" +
                "getRobjReplacement(3)#exit_02]")
            return (True, rpy.r('format(' + accVar + ')')) 
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__" +
            "getRobjReplacement(3)#exit_01]")
        return (False, None)

        
    # Should look for both @ and $, and split on the one with the highest last index, and return [-1]
    def __getFriendlyName(self, var):
        """
            Convenice method. It simple extracts the last element of a string if
            the string contains the character '$'.

            Params:
              var - StringType : A string representing a variable in R (can be any string ofcourse).

            Returns:
              The string itself if '$' is not found in the string, the last element otherwise.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getFriendlyName(2)#entry]")
        # [NOTE] Please note the fallthrough mechanism in this method.
        inType = type(var)
        if inType is types.ListType:
            if var == []:
                return "un-named"
            lastEl = var[-1]
            if type(lastEl) is types.TupleType:
                if len(lastEl) != 2:
                    return str(var)
                # Fallthough to string handling.
                var = lastEl[1]
                inType = type("string")
            elif type(lastEl) is types.StringType:
                # Fallthough to string handling.
                var = lastEl
                inType = type("string")
        if inType is types.StringType:
            if "[" in var or "]" in var:
                var = var.replace("[", "").replace("]","")
            if "$" in var:
                dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getFriendlyName(2)#exit_01]")
                return var.split("$")[-1]
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getFriendlyName(2)#exit_02]")
        return str(var)

        

    def __setDefaultMeta(self, var, typeOverride = None):
        """
            Sets default metadata based on the specs of the Python object in the Data object provided as parameter.
            This method depends on the values returned by the method __getDefaultType() if 'typeOverride' is not set.

            Params:
              var - Data object : This object contains an arbitrary python common object (list, array, integer
                                  string, boolean, complex, float, long). The object is accessed with Data.getData().

            Returns:
              A dictionary with special metadata, where default values are set if other values are not already set.
              Any existing keys whose meaning we don't know anything about, is kept.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setDefaultMeta(2/3)#entry]")
#        pass # Not yet implemented, as SciCraft currently does not have standardized metadata on its data-objects
        # Solution #1 - return manipulated dictionary.
        if type(var) is not types.InstanceType:
            raise TypeError, "__setDefaultMeta only operates on Data objects. Received " + str(type(var))
        data = var.getData()
        defType = self.__getDefaultType(data)
        # [NOTE] Override only changes 'class', not the other parts of the type definition ('is', 'mode' and 'typedef').
        if typeOverride != None:
            defType['class'] = typeOverride
        classDef = defType['class']
        defMeta = var.getMetadataSpecial()
        if type(defMeta) is not types.DictType:
            defMeta = {}
        if not defMeta.has_key(self.specialAttr):
            defMeta[self.specialAttr] = {}
        if not defMeta.has_key(self.genericAttr):
            defMeta[self.genericAttr] = {}
        if not defMeta[self.specialAttr].has_key('class'):
            defMeta[self.specialAttr]['class'] = defType

        # Used in dprints only.
        nameStr = "'" + str(var.getName()) + "'."
        # Different things to do for each type.
        if classDef == 'character' or \
            classDef == 'integer' or \
            classDef == 'numeric' or \
            classDef == 'logical' or \
            classDef == 'complex' or \
            classDef == 'array':
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setDefaultMeta(2/3)#exit_01]")
            return defMeta
        
        elif classDef == 'list':
            if not defMeta[self.specialAttr].has_key('fields'):
                    dprint(self.DEVINFO, "WARNING: Correcting meta for list object", nameStr)
                    # [NOTE] This method does not recurse... Assumes simple/consistent data.
                    fields = []
                    iterator = data.__iter__()
                    if type(data) is types.DictType:
                        names = []
                        try:
                            while 1:
                                names.append(iterator.next())
                                tmpData = Data(names[-1], data[names[-1]])
                                tmpData.setTypeSpecial({'class': "let_RPy_handle_this", 'is': "let_RPy_handle_this"})
                                fields.append(tmpData)
                        except StopIteration:
                            pass
                        # Create entry for 'names' attribute of the list (since we have a dictionary).
                        tmpData = Data('names', names)
                        tmpData.setTypeSpecial({'class': "vector", 'is': "vector"})
                        tmpData.setMetadataSpecial({self.specialAttr: {'hasNA': 0}, self.genericAttr: {}})
                        defMeta[self.genericAttr]['names'] = tmpData
                    else:
                        try:
                            tmpCounter = 1
                            while 1:
                                tmpData = Data(str(tmpCounter), iterator.next())
                                tmpData.setTypeSpecial({'class': "let_RPy_handle_this", 'is': "let_RPy_handle_this"})
                                fields.append(tmpData)
                                tmpCounter += 1
                        except StopIteration:
                            pass
                    defMeta[self.specialAttr]['fields'] = fields
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setDefaultMeta(2/3)#exit_02]")
            return defMeta
        
        elif classDef == 'vector':
            if not defMeta[self.specialAttr].has_key('hasNA'):
                dprint(self.DEVINFO, "WARNING: Correcting meta for vector object", nameStr)
                # We know nothing, so we must assume NA is present.
                defMeta[self.specialAttr]['hasNA'] = 1
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setDefaultMeta(2/3)#exit_03]")
            return defMeta
        
        elif classDef == 'data.frame':
            # Should data frames get factors or vectors (AsIs)? 
            # We go for factors first (simpler). Can easily be changed later (remember to set AsIs).
            if not defMeta[self.specialAttr].has_key('fields'):
                dprint(self.DEVINFO, "WARNING: Correcting meta for data.frame object", nameStr)
                fields = {}
                names  = []
                data = var.getData()
                # If data is a list, just add the rows. Order in frame will be as it is in the list.
                if type(data) is types.ListType:
                    for index in xrange(data):
                        strIndex = str(index +1)
                        names.append(strIndex)
                        tmpData = Data(strIndex, data[index])
                        tmpData.setMetadataSpecial(self.__setDefaultMeta(tmpData, "factor"))
                        fields[strIndex] = tmpData
                # If data is a dictionary, deal with the row names. Order is "random" (set by dictionary).
                if type(data) is types.DictType:
                    fields = {}
                    names  = []
                    for el in data.keys():
                        names.append(el)
                        tmpData = Data(el, data[el])
                        tmpData.setMetadataSpecial(self.__setDefaultMeta(tmpData, "factor"))
                        fields[el] = tmpData
                defMeta[self.specialAttr]['fields'] = fields
                defMeta[self.specialAttr]['names']  = names
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setDefaultMeta(2/3)#exit_04]")
            return defMeta    

        elif classDef == 'factor':
            if not defMeta[self.specialAttr].has_key('as.integer'):
                dprint(self.DEVINFO, "WARNING: Correcting meta for factor object", nameStr)
                data = Tools.assertList(data)
                level = 1
                index = 0
                prev = ""
                asInteger = []
                # [NOTE] Do NOT use 'transTbl' the get the levels after the for loop! (unordered is not good...)
                levels = []
                transTbl = {}
                for el in data:
                    if el not in transTbl:
                        levels.append(el)
                        asInteger.append(level)
                        transTbl[el] = level 
                        level += 1
                    else:
                        asInteger.append(transTbl[el])
                    index += 1    
                defMeta[self.specialAttr]['levels']     = levels
                defMeta[self.specialAttr]['as.integer'] = asInteger
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setDefaultMeta(2/3)#exit_05]")
            return defMeta

        elif classDef == 'matrix':
            dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__setDefaultMeta(2/3)#exit_06]")
            return defMeta

        elif classDef == 'call':
            if not defMeta[self.specialAttr].has_key('callList'):
                # Unable to handle this situation.
                raise AttributeError, "__setDefaultMeta is unable to generate metadata for R object of class 'call'. " + \
                                        "Please make sure the R object is fetched by the appropriate method."
            return defMeta

        elif classDef == ['terms', 'formula']:
            return defMeta

        raise AttributeError, "__setDefaultMeta is unable to handle data of this class: " + defType
    

    
    def __getDefaultType(self, var):
        """
            Examines a Python variable and determines what R object class the
            variable would belong in. Then a type definition dictionary with
            four keys is genrated and returned: 'class', 'is', 'mode' and 
            'typeof'. These keys are the names of the corresponding functions
            in R that generate values for these under normal operation.
            The type definition is generated without the help of any metadata.
            This limits the number of different R classes that are returned
            by this method.
            
            Params:
                var - object : A Python object, whose R type we want to get.

            Returns:
                A dictionary representing a type definition. This has four
                keys: 'class', 'is', 'mode' and 'typeof'. The values here
                will be the same as the R function with these names return.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getDefaultType(2)#entry]")
        typ = type(var)
        typeDef = {'class': None, 'is': None, 'mode': None, 'typeof': None} 
        if typ is types.StringType:
            typeDef['class'] = typeDef['mode'] = typeDef['typeof'] = "character"
            typeDef['is']                                          = ["character", "vector"]
        elif typ is types.IntType:
            # [NOTE] Handling as R class 'integer'. Can also be handled as 'numeric'.
            typeDef['class'] = typeDef['typeof'] = "integer"
            typeDef['mode']                      = "numeric"
            typeDef['is']                        = ["integer", "vector", "numeric"]
        elif typ is types.FloatType:
            typeDef['class']  = typeDef['mode'] = "numeric"
            typeDef['typeof']                   = "double"
            typeDef['is']                       = ["numeric", "vector"]
        elif typ is types.LongType:
            # [NOTE] Handling as R class 'numeric', mainly due to the convertLongList method.
            #        Long must be converted, or else RPy segfaults.
            typeDef['class'] = typeDef['mode'] = "numeric"
            typeDef['typeof']                  = "double"
            typeDef['is']                      = ["numeric", "vector"]
        elif typ is types.ListType:
            typeDef['class'] = typeDef['mode'] = typeDef['typeof'] = "list"
            typeDef['is']                                          = ["list", "vector"]
        elif typ is types.DictType:
            typeDef['class'] = typeDef['mode'] = typeDef['typeof'] = "list"
            typeDef['is']                                          = ["list", "vector"]
        elif typ is types.NoneType:
            # 'None' will now be 'FALSE' in R.
            typeDef['class'] = typeDef['mode'] = typeDef['typeof'] = "logical"
            typeDef['is']                                          = ["logical", "vector"]
            # Could also be handled as R 'NULL'.
            # typeDef['class'] = typeDef['mode'] = typeDef['typeof'] = "NULL"
            # typeDef['is']                                          = ["NULL", "OptionalFunction"]
        elif typ is types.ComplexType:
            typeDef['class'] = typeDef['mode'] = typeDef['typeof'] = "complex"
            typeDef['is']                                          = ["complex", "vector"]
        elif typ is types.TupleType:
            typeDef['class'] = typeDef['mode'] = typeDef['typeof'] = "list"
            typeDef['is']                                          = ["list", "vector"]
        elif typ is Numeric.ArrayType:
            typeDef['class'] = "matrix"
            typeDef['is']    = ["matrix", "structure", "array"]
            # Data must be inspected to determine 'mode' and 'typeof'. Can we use something as default?
            # Check one or a few data elements? Check all (assert consistency)?
            # Value not important for R, as the mode and typeof will "self-correct", but might be
            # important in Python, as decisions can be made based on them.
            typeDef['mode'] = "set_value_here!"
            typeDef['typeof'] = "set_value_here!"

            # This can also be handled as an array.
            # typeDef['class'] = "array"
            # typeDef['is']    = ["array", "structure"]
            # Data must be inspected to determine 'mode' and 'typeof'. Can we use something as default?
            # Check one or a few data elements? Check all (assert consistency)?
            # typeDef['mode'] = 
            # typeDef['typeof'] =
        else:
            raise TypeError, "[__getDefaultType#001] Unsupported data type. Got %s." % str(typ)

        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.__getDefaultType(2)#exit]")
        return typeDef


        
    def assignVar(self, varname, varvalue):
        """This method assigns the specified data to the specified variable in R.
            It also takes care of issues which RPy is unable to handle automatically:
              - converting longs to floats.

            Params:
              varname  - string : The name of the variable in R to assign the data to.
              varvalue -        : The data to assign to the variable in R. Must be of a type
                                  RPy is able to handle.

           Returns:
             Nothing.
        """
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.assignVar(3)#entry]")
        try:
            rpy.r.assign(varname, varvalue)
        except rpy.RException, e:
        # Found long. RPy does not convert long to R. Hence, if
        # a long-type is within the data, it needs to be converted
        # to float before sent to R.

        # TEMPORARY - make recursive.
            if (e[0] == "cannot convert from type 'long'"):

                if type(varvalue) is types.LongType:
                # data is long-type. Converting to float.
                    varvalue = float(data)

                elif str(type(varvalue)) == "<type 'array'>":
                    varvalue = Numeric.array(varvalue, 'd')

                elif type(varvalue) is types.ListType:
                    varvalue = self.convertLongList(varvalue)
                
                # data is list-type. Need to locate the long(s).
                rpy.r.assign(name, varvalue)
            else:
                raise ZPluginError, "An error occured when assigning variable to R.", e

        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.assignVar(3)#exit]")


            
    def convertLongList(self, lst):
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.convertLongList(2)#entry]")
        for x in xrange(len(lst)):
            if type(lst[x]) is types.LongType:
                dprint(self.DEBUG, "found float in list")
                lst[x] = float(lst[x])
            elif type(lst[x]) is types.ListType:
                dprint(self.DEBUG, "found nested list, recursing..")
                lst[x] = self.convertLongList(lst[x])
            elif str(type(lst[x])) == "<type 'array'>" and lst[x].typecode() == 'O':
                dprint(self.DEBUG, "found array, looking for longs...")
                try:
                    lst[x] = Numeric.array(lst[x], 'd')
                except:
                    dims = Tools.arrayDims(lst[x])
                    if dims == 1:
                        for i in xrange(len(lst[x])): 
                            if type(lst[x][i]) is types.LongType: 
                                dprint(self.DEBUG, "found long in 1 dim array, converting..")
                                lst[x][i] = float(lst[x][i])
                    elif dims == 2:
                        for i in range(len(lst[x])):
                            for j in range(len(lst[x][0])):
                                if type(lst[x][i,j]) is types.LongType:
                                    dprint(self.DEBUG, "found long in 2 dim array, converting..")
                                    lst[x][i,j] = float(lst[x][i,j])
        dprint(self.TRACE, "[Library.Plugins.RObjectHandler.convertLongList(2)#exit]")
        return lst
