# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.


class MultipleOptions:
    def __init__(self,parent):
        self.type = ""
        self.defaults = []
        self.options = []
        self.parent = parent

    def setType(self,type):
        """
        Params:
               - type : String - Declaring the type of visualisation preferred.
               """
        
        self.type = type

    def getType(self):
        """
        Return:
               - String - Representing the preferred visualisation method.
               """
        
        return self.type

    def addDefault(self,default):
        """
        Params:
               - default : String - The value to be set to the parameter object.
        Sets the value of the parameter.
        """
        
        self.parent.value += "," + str(default)

    def getDefaults(self):
        """
        Return:
               - String - Comma separated  list containing all the values that this parameter has set.
        """
        
        return self.parent.value

    def addOption(self,option):
        """
        Params:
               - option : String - A value that the user may choose for a list
        Adds an option to the option list.
        """
        
        self.options.append(option)

    def getOptions(self):
        """
        Return:
              - List[String] - All the options the user may choose from
        """
        
        return self.options

    def toXML(self):
        """
        Return:
               - List[String] - List of lines containg XML instance of this instance
        Makes XML-code of this object      """

        returnlist = []
        returnlist.append("<multiple type=\"" + self.getType() + "\">")
        for option in self.getOptions():
            default = ""
            if self.getDefaults().count(option):
                default = " default=\"1\""
            returnlist.append("\t<option" + default + ">" + option + "</option>")
        returnlist.append("</multiple>")
        return returnlist

