# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.


class MultipleRange:
    def __init__(self,parent):
        self.type = ""
        #self.default = ""
        self.min = 0
        self.max = 0
        self.step = 1
        self.parent = parent
        
    def setType(self,type):
        """
        Params:
               - type : String - Declaring the type of visualisation preferred.
               """
        
        self.type = type

    def getType(self):
        """
        Return:
               - String - Representing the preferred visualisation method.
               """
        
        return self.type

    def setDefault(self,default):
        """
        Params:
               - default : String - The value to be set to the parameter object.
        Sets the value of the parameter.
        """
 
        self.parent.value = str(default)

    def getDefault(self):
        """
        Return:
               - List[String] - List containing all the values that this parameter has set.
        """
        
        return self.parent.value

    def setMin(self,min):
        """
        Params:
               - min : String - The minimal value of this parameter
               """
        
        self.min = min

    def getMin(self):
        """
        Return:
               - String - The minimal value of this parameter
        """
        
        return self.min

    def setMax(self,max):
        """
        Params:
               - min : String - The maximal value of this parameter
               """
      
        self.max = max

    def getMax(self):
        """
        Return:
               - String - The maximal value of this parameter
        """

        return self.max

    def setStep(self,step):
        """
        Params:
               - step : String - The steps between the automatically made option values
               """
        
        self.step = step

    def getStep(self):
        """
        Return:
               - String - The steps between the automatically made option values
               """

        return self.step

    def toXML(self):
        """
        Return:
               - List[String] - List of lines containg XML instance of this instance
        Makes XML-code of this object      """

        returnlist = []
        returnlist.append("<multiple type=\"" + self.getType() + "\">")
        default = self.getDefault()
        if default:
            default = " default=\"" + default + "\""
        returnlist. append("\t<range max=\"" + str(self.getMax()) + "\" min=\"" + str(self.getMin()) + "\" step=\"" + str(self.getStep()) + "\"" + default + "\/>")
        returnlist.append("</multiple>")
        return returnlist
