# Should be run with main scicraft directory as working directory
# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.

class Filter:
    """Small container class representing a reference to a colum in a data
    set.
    """
    def __init__(self, source, index):
        """Constructs a filter by setting a data source object, and
        indicating the index og the row or column.
        """
        
        self._source = source
        self._index = index
        

    def get_values(self, indices = []):
        """Retrieves the actual data filtered by this this filter.
        """
        pass

    def get_source(self):
        return self._source

    def get_axis_label(self):
        """Returns the label of the variable (or axis) that this row/column
        represents in the original data set. May return the empty string,
        meaning that the axis has no label yet.
        """
        pass

    def get_object_labels(self):
        """Returns the labels for all the values in this row/column
        of data. May return an empty list, menaning that the labels have not
        been set.
        """
        pass

    def index_within_bounds(self):
        """Checks if this filter is within the specified bounds of the
        data object from which it gets the actual data.
        """
        pass

    def rows_changed(self, marked_rows = [], unmarked_rows = []):
        self._source.rows_changed(marked_rows, unmarked_rows)

class RowFilter(Filter):
    """This is a filter for where a row of the original source
    yields a column of the filtered data set.
    """

    def index_within_bounds(self):

        if self._index >= len(self._source.get_data()):
            return False
        else:
            return True
    
    def get_values(self, indices = []):

        source_data = self._source.get_data()
        result_data = []

        if not indices:
            try:
                return source_data[self._index]
            except IndexError:
                return []

        for value in indices:
            try:
                result_data.append(source_data[self._index][value])

            except IndexError:
                break

        return result_data

    def get_axis_label(self):
        
        labels = self._source.get_row_labels()
        try:
            return str(labels[self._index])
        except IndexError:
            return ""

    def get_object_labels(self):
        
        return self._source.get_column_labels()

    def rows_changed(self, marked_rows = [], unmarked_rows = []):
        """A row has been marked in the
        Filtered matrix. The index of the marked row will
        be translated to the static index of the RowFilter,
        since there is only one row used from the original MatrixDataSet.
        This soultion is not verifed to be acurate in an academical sence, i.e.
        this method will be removed if it makes no mathematical sense
        to mark rows in this fashion.
        """
        # self._source.row_marked(self._index)
        pass

class ColumnFilter(Filter):
    
    def index_within_bounds(self):
        
        if (len(self._source.get_data()) < 1 or
            self._index >= len(self._source.get_data()[0])):
            return False
        else:
            return True

    def get_values(self, indices = []):

        source_data = self._source.get_data()
        result_data = []

        if not indices:
            for row in source_data:
                try:                    
                    result_data.append(row[self._index])
                except IndexError:
                    break
            return result_data
        
        for value in indices:
            try:
                result_data.append(source_data[value][self._index])
                
            except IndexError:
                break
                
        return result_data

    def get_axis_label(self):

        labels = self._source.get_column_labels()
        try:
            return str(labels[self._index])
        except IndexError:
            return ""

    def get_object_labels(self):

        return self._source.get_row_labels()
