# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.

from System.FileHandlers.Input.FileReader import FileReader
from Library.Util.lists import TokenFile
from System.Data.Data import Data

class TextFileReader(FileReader):
    """This class is able to read data in ascii-format from a file"""

    name = "TextFileReader"
    supportedFileExtensions = ["txt","data"]
    description = "ASCII file format."
    variable_name_support = False
    fileDescription = "Textfile"

    
    def __init__(self, name="TextFileReader"):
        self.string_list = []
        self.portName = "text"
        self.portDescription = "One or more strings."

    def read_data(self, file_path, split_on = ['\n'], col_split=None):
        self.string_list = list(TokenFile(file_path, split_on=split_on,
                                          col_split=col_split,
                                          parse_numbers=True))
        return [Data(self.portName, self.string_list)]

    def get_default_output_ports(self):
        return ((self.portName, "", self.portDescription),)

    def get_variable_info(self, filename, variable_name = None, node = None):
        
        if node:
            filename = self.convert_to_absolute_filepath(filename, node.get_module_diagram().getFilename())
        
            
        self.string_list = list(TokenFile(filename, split_on = ['\n']))
        variable_info = {}
        variable_info[self.portName] = str(len(self.string_list))

        if variable_name:
            if variable_name == self.portName:
                return variable_info[variable_name]
            else:
                return ""
        else:
            return variable_info
