# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.

from System.FileHandlers.Output.FileWriter import FileWriter
from Library.Util.rtester import RTester
from System.Exceptions import ZPluginError

        
class RFileWriter(FileWriter):
    """This class is responsible for writing R-files to disk. It inherits its
       methods from FileWriter, but overwrites all methods."""

    supportedFileExtensions = ["RData"]
    fileDescription = "R"
    description = "GNU R file format."
    name = "RFileWriter"
    
        
    def write_data(self, data, destinationPath):
        """
            Params:
                - data              : List[Data]    - A List with Data objects
                - destinationPath   : String        - The path and name for 
                                                      the file being written.
            
            Exception:
                - IOError - Cannot write to file. Possible causes:
                    * No write priveleges at selected path.
                    * Not enough free space on disk.

            This is the method that actually writes the data to an R-object 
            ascii-file.
        """
        
        """These imports take time, so we import them only when we have used for them. """
        RTester.r_install_test()
        import rpy
        from Library.Plugins.RPluginFiles import RObjectHandler

        rpy.r.rm(list = rpy.r.ls()) # Removing current global variables in R.

        robjHandler = RObjectHandler.RObjectHandler()
        for x in data:
            robjHandler.setRVar(x)
        try:
            # Saving all global R-variables to disk.
            rpy.r.save(list = rpy.r.ls(), file = destinationPath) 
        except RException, e: 
            print e, "\nR could not write data to file " + destinationPath +\
                ". Check permissions and storage space."
