# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.
  


"""Simple datawrapper class which encompasses all information needed
to create a FunctionNode. Created at startup by XMLFunction from the
XML files.

All values stored are of the type needed by Node. The functions which
take several arguements have values stored as tuples. Easiest way to
call these functions afterwards is thus: apply(function,
functionargs)
"""

from System.Data.FunctionNode import FunctionNode


class Function:
    
    def __init__(self):
        """
        Inits variables """
        
        self.name = ""
        self.nodeRepName = ""
        self.file = ""
        self.gui = ""
        self.description = ""
        self.plugin = None
        self.parameters = []
        self.inputs = []
        self.outputs = []
        self.filename = ""
        self.htmlLink = None
        self.menuAddress = ""
        self.sub_objects = {}
        self.doc_path = ""
        self.icon = ""

    def createFunctionNode(self, pluginrepository):

        newNode = FunctionNode(self.getName(), pluginrepository)
        newNode.setPluginType(self.getPlugin())

        # setting the node used in the node repository
        newNode.setNodeRepName(self.getNodeRepName())

        # filepath = absolute path to filename
        newNode.setFileName(self.getFileName())

        # setting menuAdress
        newNode.setMenuAddress(self.getMenuAddress())

        for values  in self.getNodeInputPorts():
            isNamed = values[3]
            values = values[:3]
            port = apply(newNode.add_input_port,values)
            port.setIsNamed(isNamed)

        for values in self.getNodeOutputPorts():
            apply(newNode.add_output_port,values)

        params = self.getParameters()
        for values in params:
            param = apply(newNode.addParameter,values[:3])
            rest = values[3]
            param.setRestrictions(max = rest[0], min = rest[1], default = rest[2], step = rest[3], values = rest[4])
            param.setIsNamed(values[4])

        # setting description
        newNode.setDescription(self.getDescription())           

        # setting sub object
        newNode.set_sub_objects(self.sub_objects)
        
        # setting icon
        if self.icon != None:
            newNode.set_icon(self.icon)

        return newNode
    

    def setName(self,name):
        self.name = str(name)
    
    def getName(self):
        return self.name

    def setPlugin(self,plugin):
        self.plugin = str(plugin)

    def getPlugin(self):    
        return self.plugin

    def setNodeRepName(self,name):
        self.nodeRepName = str(name)
        
    def getNodeRepName(self):
        return self.nodeRepName


    def setFileName(self,filename):
        self.filename = filename

    def getFileName(self):
        return self.filename

    def setDescription(self,description):
        self.description = str(description)

    def getDescription(self):
        return self.description

    def addParameter(self,parameter): 
        self.parameters.append(parameter)

    def getParameters(self):
        return self.parameters
    
    def add_input_port(self, values):
        self.inputs.append(values)

    def getNodeInputPorts(self):
        return self.inputs
    
    def add_output_port(self, values):
        self.outputs.append(values)

    def getNodeOutputPorts(self):
        return self.outputs

    def setHtmlLink(self,link):
        self.htmlLink = link

    def getHtmlLink(self):
        if not self.htmlLink: return self.htmlLink
        return str(self.htmlLink)
    
    def setMenuAddress(self,address):
        self.menuAddress = address
    
    def getMenuAddress(self):
        if not self.menuAddress: return self.menuAddress
        return str(self.menuAddress)

    def set_sub_objects(self, objects):
        self.sub_objects = objects
    
    def get_sub_objects(self):
        return self.sub_objects

    def set_doc_path(self, path):
        self.doc_path = path

    def get_doc_path(self):
        return self.doc_path

    def set_icon(self, icon):
        self.icon = icon
        
    def get_icon(self):
        return self.icon