# SciCraft - An interactive graphics front end to advanced data mining
# Copyright (C) 2002-2005 SciCraft Development Team
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# As a special exception, SciCraft Development Team gives permission
# to link this program with  Qt non-commercial edition, and distribute
# the resulting executable, without including the source code for the
# Qt non-commercial edition in the source distribution.



# DEBUG PRINTING:

import sys
import thread

flags = lambda : None

# System-wide flags
flags.THREADING         = 0 # Show info when threads are created/stopped etc.
                           
# Component specific flags.
flags.GUI = lambda : None
flags.GUI.PLOTS = lambda : None

flags.GUI.PLOTS.PARSING = 0 # Messages when building the plotlist from files
flags.GUI.PLOTS.BUILD   = 0 # When the menus for the GUI is built
flags.GUI.PLOTS.FETCH   = 0 # When a plot type is requested from the user
flags.GUI.PLOTS.OPTIONS = 0 # When a user fiddles with the plot option dialogs
flags.GUI.PLOTS.SAVING  = 0 # When a user tries to save a plot to a file
flags.GUI.PLOTS.SETUP   = 0 # When a user is setting up the plot window
flags.GUI.PLOTS.RUN     = 0 # When a plot is running or about to run

flags.GUI.CREATE = lambda : None
flags.GUI.CREATE.NODE = 0 # Message when creating a new node
flags.GUI.CREATE.LINK = 0 # Message when creating a new link
flags.GUI.CREATE.DIAGRAM = 0 # Message when creating a new diagram

flags.GUI.CANVAS = lambda : None
flags.GUI.CANVAS.CANVASVIEW = 0 # Debug messages from CanvasView

flags.GUI.DELETE = lambda : None
flags.GUI.DELETE.LINK = 0
flags.GUI.DELETE.NODE = 0

flags.GUI.NODELISTDLG = lambda : None
flags.GUI.NODELISTDLG.LOADLIST = 0 # Message when nodelistdialog reloads its list
flags.GUI.NODELISTDLG.UPDATE   = 0 # Messages when nodelistdialog is updated

flags.GUI.MOVE = lambda : None
flags.GUI.MOVE.NODE = 0 # Message when moving a node
flags.GUI.MOVE.LINK = 0 # Message when moving a link

flags.SYSTEM = lambda : None
flags.SYSTEM.NODESTATUSMODEL = lambda : None
flags.SYSTEM.NODESTATUSMODEL.ACTIONS     = 0
flags.SYSTEM.NODESTATUSMODEL.MODEL       = 0 # Prop. to number of nodes in MD.
flags.SYSTEM.DATA = lambda : None
flags.SYSTEM.DATA.NODE                   = 0
flags.SYSTEM.DATA.NODEPORT               = 0

flags.SYSTEM.DATA.MODULEDIAGRAM = lambda : None
flags.SYSTEM.DATA.MODULEDIAGRAM.RUNNINGDIAGRAM  = 0 # basic started msg's & runtime
flags.SYSTEM.DATA.MODULEDIAGRAM.RUNTIME         = 0 # Show the time taken to run MD.
flags.SYSTEM.DATA.MODULEDIAGRAM.REMOVENODE      = 0 # invoked everytime a node is removed
flags.SYSTEM.DATA.MODULEDIAGRAM.USERINFO = 0
flags.SYSTEM.EVENTSERVER                 = lambda : None
flags.SYSTEM.EVENTSERVER.RECEIVE         = 0
flags.SYSTEM.EVENTSERVER.SEND            = 0
flags.SYSTEM.NODEREPOSITORY              = 0
flags.SYSTEM.PLUGIN = lambda : None
flags.SYSTEM.PLUGIN.PLUGINREPOSITORY     = 0
flags.SYSTEM.XMLREADER                   = 0

flags.LIBRARY = lambda : None
flags.LIBRARY.PLUGINS = lambda : None
flags.LIBRARY.PLUGINS.OCTAVEPLUGIN                     = 0
flags.LIBRARY.PLUGINS.OCTPY                            = 0
flags.LIBRARY.PLUGINS.RPLUGIN                          = 0
flags.LIBRARY.PLUGINS.PYTHONPLUGIN                     = 0
flags.LIBRARY.FILEREADERS = lambda : None
flags.LIBRARY.FILEREADERS.ERRORS                       = 0
flags.LIBRARY.FILEREADERS.OCTAVEFILEREADER             = 0
flags.LIBRARY.FILEREADERS.BASEFILEREADER = lambda : None
flags.LIBRARY.FILEREADERS.BASEFILEREADER.USERINFO      = 0       # Informatino the user should see.
flags.LIBRARY.FILEREADERS.BASEFILEREADER.USERERRORMSGS = 0  # Error messages the user should see.
flags.LIBRARY.FILEREADERS.BASEFILEREADER.ERROR         = 0          # Other errors, more technical.

flags.LIBRARY.NODES = lambda : None
flags.LIBRARY.NODES.EDITNODES = lambda : None
flags.LIBRARY.NODES.EDITNODES.EDITNODE = lambda : None
flags.LIBRARY.NODES.EDITNODES.EDITNODE.INFO             = 0          # Information the user should see.
flags.LIBRARY.NODES.EDITNODES.EDITNODE.USERERRORMSGS    = 1 # Error messages the user should see.
flags.LIBRARY.NODES.EDITNODES.EDITNODE.COMMANDS         = 0      # Errors from the available commands (__cmd_name methods).
flags.LIBRARY.NODES.EDITNODES.EDITNODE.EXECUTE          = 0       # Errors from the execute() method.        
flags.LIBRARY.NODES.EDITNODES.EDITNODE.GENERAL          = 0       # General error messages.
flags.LIBRARY.NODES.EDITNODES.EDITNODE.DEBUGINFO        = 0     # Debugging information (content of vars etc)
flags.LIBRARY.NODES.EDITNODES.MOLECULEEDITNODE = lambda : None
flags.LIBRARY.NODES.EDITNODES.MOLECULEEDITNODE.INFO     = 0          # Information the user should see.
flags.LIBRARY.NODES.EDITNODES.MOLECULEEDITORNODE = lambda : None
flags.LIBRARY.NODES.EDITNODES.MOLECULEEDITORNODE.INFO   = 0          # Information the user should see.

flags.LIBRARY.UTIL = lambda : None
flags.LIBRARY.UTIL.NULLEVENTSERVER                      = 0
flags.LIBRARY.UTIL.INTERRUPTIBLECOMMAND                 = 0

flags.USERMESSAGES = lambda : None # General information which the user should see. Could be anything
flags.USERMESSAGES.GUI = 0

flags.UTIL = lambda : None
flags.UTIL.MOLECULAR = lambda : None
flags.UTIL.MOLECULAR.INFO = 0          # Information the user should see.
flags.UTIL.MOLECULAR.DEBUGINFO = 0
flags.UTIL.CONFIG = lambda : None
flags.UTIL.CONFIG.INFO = 0 # Information the user should see.

flags.TESTNODES = lambda : None
flags.TESTNODES.PARAMNODE = 0

COLORS_ENABLED = False # Change to True to enable colors for other threads

_colors = ('off', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'white')
colors = dict([(_colors[i], "\033[0;%dm" % (30 + i)) for i in range(len(_colors))])
reset = "\033[0m"

def dprint(flag,*args):
    if flag:
        for arg in args:
            ident = thread.get_ident()
            if ident != 16384 and COLORS_ENABLED:
                sys.stdout.write(colors['yellow'])
            if isinstance(arg, type(unicode())):
                if COLORS_ENABLED:
                    print ident, unicode(arg) + reset,
                else:
                    print ident, unicode(arg),
            else:
                if COLORS_ENABLED:
                    print ident, str(arg) + reset,
                else:
                    print ident, str(arg),
        print 

