FilterRG <-function(RG,FilterInfo,GeneStatus,BothChannals=TRUE,Flags="Remove flagged spots",ControlSpots="Weigh down controls",FilterMethod="Remove spots",NoiseWeights="No noise weights",RedMinIntensity=200,GreenMinIntensity=200,RedBgRatio=2,GreenBgRatio=2,SaturationLimit=0.30,PixelLimit= 40,MaxMdev=2){

library(limma)
#######Functions to handle control spots###############
##################################################
weighdownControls<-function(RG,GeneStatus="cDNA") {

gene_index<- RG$genes$Status == GeneStatus

RG$weights[!gene_index,]=10^-6
RG

}

get_genes<-function(RG,GeneStatus="cDNA") {

gene_index<- RG$genes$Status == GeneStatus

RG<-RG[gene_index,]
RG

#make sure integrity of replicates
}

FindGeneStatus <- function(RG) {
 #find gene code
 stati <- unique(RG$genes$Status)
 counts <- vector("numeric",length(stati))
 index <- 0

  for (sta in stati) {
   index <- index+1
   ind <- RG$genes$Status==sta
   counts[index]<-sum(ind)
   }

 ind<-which.max(counts)
 GeneStatus<-stati[ind]
 GeneStatus
}

#######################################################
######################################################


if (is.null(RG$weights)){
RG$weights <- matrix(1,nrow=dim(RG$R)[1],ncol=dim(RG$R)[2])
}

 #GeneStatus<-FindGeneStatus(RG)
 if (ControlSpots=="Remove controls"){
  RG<- get_genes(RG,GeneStatus=GeneStatus)
 }
#}

#set up initial information about the dataset
n_arrays<-dim(RG)
n_spots <-n_arrays[1]
n_arrays<-n_arrays[2]


#Detect bad spots
#If both channals are saturated

sat_index <- (FilterInfo$Gsat > SaturationLimit & FilterInfo$Rsat > SaturationLimit)
size_index <- FilterInfo$FGpixels < PixelLimit

#If both cannals are low

g_index <- RG$G < GreenMinIntensity | RG$G < RG$Gb*GreenBgRatio 
r_index <- RG$R < RedMinIntensity | RG$R < RG$Rb*RedBgRatio 

if (Flags == "Remove spots with negative flags") {
 w_index <- FilterInfo$Flags < 0
 RG$G[w_index]<-NA
 RG$R[w_index]<-NA
}

if (Flags == "Reduce spot weight to 10^-6") {
 w_index <- FilterInfo$Flags < -0.1
 RG$weights[w_index]<- 10^-6
}

#Signal to noise for both channals
#Rs2n <- (sqrt(FilterInfo$R_SD^2))/RG$R
#Gs2n <- (sqrt(FilterInfo$G_SD^2))/RG$G

dM = sqrt((1/RG$R^2)*(FilterInfo$R_SD^2)+(1/RG$G^2)*(FilterInfo$G_SD^2))

s2n_index <- dM > MaxMdev

 if (BothChannals=="TRUE") {
  spot_index <- g_index & r_index | sat_index | s2n_index #|size_index
 }
 else{
  spot_index <- g_index | r_index | sat_index | s2n_index #|size_index 
 }

#set to NA or use weights
if (FilterMethod=="Remove spots"){
 RG$G[spot_index]<-NA
 RG$R[spot_index]<-NA
}

if(NoiseWeights == "Weigh by 1/(s^2+0.5)"){
RG$weights <- 1/(dM+0.5)
#ind <-RG$weights > 1
#RG$weights[ind] <- 1
}

if(NoiseWeights == "Weigh by 1/s^2"){
RG$weights <- 1/(dM)
#ind <-RG$weights > 1
#RG$weights[ind] <- 1
}

if (FilterMethod == "Reduce spot weight to 10^-6"){
RG$weights[spot_index]<-10^-6
}

if (FilterMethod== "Reduce spot weight to 0.1"){
RG$weights[spot_index]<-0.1}

if (ControlSpots=="Weigh down controls"){
  RG<-weighdownControls(RG,GeneStatus=GeneStatus)
 }




return(list(RGfiltered=RG))#,dM = dM))
}

